/*----------------------------------------
description file: beetles.mLD9013106

% file beetles.mLD90
% weibull regression model
% new parametrisation : t2, LD90
resp y; 
varind CS2,n;
aux p, eta; 
var v;
parresp LD90,t2;
subroutine;
begin 
eta = log(-log(0.1))+ t2* (CS2 - LD90);
p=1-exp(-exp(eta));
y = n*p;
v = n*(1-p)*p;
end 
----------------------------------------*/
#include <stdio.h>
#include <math.h>
#include "nltypes.h"
#include "nlcodes.h"
#include "nlmacros.h"
#include "nlfonc.h"

short int calcf_(nbt, nbg, nbl, nbc, t, g, x, f, df, le, ie)
      short int nbt, nbg, nbl, nbc;
      double *t, *g, **x, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
double a[2], da[2][2];
ECRTRACE("calcf");
for (i=0; i<nbl; i++)
  {
    res= (myln(-(myln((double)1.00000000e-01)))) + ((t[1]) * (
      (x[i][0]) - (t[0])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[0]= res;
    res= ((double)1) - ((double)myexp(-((double)myexp(
      a[0]))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[1]= res;
    res= (x[i][1]) * (a[1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=F;
        return(ERRMATH);
      }
    f[i]= res;
    res= (t[1]) * (-((double)1));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][0]= res;
    res= (x[i][0]) - (t[0]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][1]= res;
    res= -((-((da[0][0]) * ((double)myexp(a[0])))) * ((double)myexp(
      -((double)myexp(a[0])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][0]= res;
    res= -((-((da[0][1]) * ((double)myexp(a[0])))) * ((double)myexp(
      -((double)myexp(a[0])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][1]= res;
    res= (x[i][1]) * (da[1][0]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][0]= res;
    res= (x[i][1]) * (da[1][1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][1]= res;
  }
return(OK);
}

short int calcv_(nbt, nbb, nbg, nbl, nbc, t, b, g,
                f, df, x, v, dtv, dbv, le, ie)
      short int nbt, nbb, nbg, nbl, nbc;
      double *t, *b, *g, **x, *v, **dtv, **dbv, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
double a[2], da[2][2];
ECRTRACE("calcv");
for (i=0; i<nbl; i++)
  {
    res= (myln(-(myln((double)1.00000000e-01)))) + ((t[1]) * (
      (x[i][0]) - (t[0])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[0]= res;
    res= ((double)1) - ((double)myexp(-((double)myexp(
      a[0]))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[1]= res;
    res= ((x[i][1]) * (((double)1) - (a[1]))) * (
      a[1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=V;
        return(ERRMATH);
      }
    v[i]= res;
    res= (t[1]) * (-((double)1));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][0]= res;
    res= (x[i][0]) - (t[0]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][1]= res;
    res= -((-((da[0][0]) * ((double)myexp(a[0])))) * ((double)myexp(
      -((double)myexp(a[0])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][0]= res;
    res= -((-((da[0][1]) * ((double)myexp(a[0])))) * ((double)myexp(
      -((double)myexp(a[0])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][1]= res;
    res= (((x[i][1]) * (((double)1) - (a[1]))) * (
      da[1][0])) + (((x[i][1]) * (-(da[1][0]))) * (a[1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][0]= res;
    res= (((x[i][1]) * (((double)1) - (a[1]))) * (
      da[1][1])) + (((x[i][1]) * (-(da[1][1]))) * (a[1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][1]= res;
  }
return(OK);
}

calcodes_()
{}

calcjac_()
{}

calcphi_()
{}
