/*----------------------------------------
description file: corti.mpois413212

% poisson regression model
resp cpm;
var v; 
varind dose; 
parresp n,d,a,b,g;
pbisresp minf,pinf;
parvar tau;
subroutine;
begin 
cpm= if dose <= minf then d else
      if dose >= pinf then n else
      n+(d-n)*exp(-g*log(1+exp(a+b*log10(dose))))
    fi fi;
v = cpm+cpm**2/tau;
end 

----------------------------------------*/
#include <stdio.h>
#include <math.h>
#include "nltypes.h"
#include "nlcodes.h"
#include "nlmacros.h"
#include "nlfonc.h"

short int calcf_(nbt, nbg, nbl, nbc, t, g, x, f, df, le, ie)
      short int nbt, nbg, nbl, nbc;
      double *t, *g, **x, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
ECRTRACE("calcf");
for (i=0; i<nbl; i++)
  {
    res= ((x[i][0]) <= (g[0])) ? (t[1]) : (((x[i][0]) >= (
      g[1])) ? (t[0]) : ((t[0]) + (((t[1]) - (t[0])) * (
      (double)myexp((-(t[4])) * (myln((
      (double)1) + ((double)myexp((t[2]) + ((t[3]) * (
      mylog(x[i][0]))))))))))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=F;
        return(ERRMATH);
      }
    f[i]= res;
    res= ((x[i][0]) <= (g[0])) ? ((double)0) : (
      ((x[i][0]) >= (g[1])) ? ((double)1) : ((
      (double)1) + ((-((double)1)) * (
      (double)myexp((-(t[4])) * (myln((
      (double)1) + ((double)myexp((t[2]) + ((t[3]) * (
      mylog(x[i][0]))))))))))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][0]= res;
    res= ((x[i][0]) <= (g[0])) ? ((double)1) : (
      ((x[i][0]) >= (g[1])) ? ((double)0) : ((double)myexp(
      (-(t[4])) * (myln(((double)1) + ((double)myexp(
      (t[2]) + ((t[3]) * (mylog(x[i][0]))))))))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][1]= res;
    res= ((x[i][0]) <= (g[0])) ? ((double)0) : (
      ((x[i][0]) >= (g[1])) ? ((double)0) : (((
      t[1]) - (t[0])) * (((-(t[4])) * (mydiv((double)myexp(
      (t[2]) + ((t[3]) * (mylog(x[i][0])))), (
      (double)1) + ((double)myexp((t[2]) + ((t[3]) * (
      mylog(x[i][0])))))))) * ((double)myexp((-(t[4])) * (
      myln(((double)1) + ((double)myexp((t[2]) + (
      (t[3]) * (mylog(x[i][0]))))))))))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][2]= res;
    res= ((x[i][0]) <= (g[0])) ? ((double)0) : (
      ((x[i][0]) >= (g[1])) ? ((double)0) : (((
      t[1]) - (t[0])) * (((-(t[4])) * (mydiv((mylog(x[i][0])) * (
      (double)myexp((t[2]) + ((t[3]) * (mylog(x[i][0]))))), 
      ((double)1) + ((double)myexp((t[2]) + ((t[3]) * (
      mylog(x[i][0])))))))) * ((double)myexp((-(t[4])) * (
      myln(((double)1) + ((double)myexp((t[2]) + (
      (t[3]) * (mylog(x[i][0]))))))))))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][3]= res;
    res= ((x[i][0]) <= (g[0])) ? ((double)0) : (
      ((x[i][0]) >= (g[1])) ? ((double)0) : (((
      t[1]) - (t[0])) * ((-(myln(((double)1) + (
      (double)myexp((t[2]) + ((t[3]) * (mylog(x[i][0])))))))) * (
      (double)myexp((-(t[4])) * (myln((
      (double)1) + ((double)myexp((t[2]) + ((t[3]) * (
      mylog(x[i][0]))))))))))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][4]= res;
  }
return(OK);
}

short int calcv_(nbt, nbb, nbg, nbl, nbc, t, b, g,
                f, df, x, v, dtv, dbv, le, ie)
      short int nbt, nbb, nbg, nbl, nbc;
      double *t, *b, *g, **x, *v, **dtv, **dbv, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
ECRTRACE("calcv");
for (i=0; i<nbl; i++)
  {
    res= (f[i]) + (mydiv(mypow(f[i], (double)2), 
      b[0]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=V;
        return(ERRMATH);
      }
    v[i]= res;
    res= (df[i][0]) + (mydiv((((double)2) * ((mypow(
      f[i], ((double)2) - ((double)1))) * (
      df[i][0]))) * (b[0]), mypow(b[0], 
      (double)2)));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][0]= res;
    res= (df[i][1]) + (mydiv((((double)2) * ((mypow(
      f[i], ((double)2) - ((double)1))) * (
      df[i][1]))) * (b[0]), mypow(b[0], 
      (double)2)));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][1]= res;
    res= (df[i][2]) + (mydiv((((double)2) * ((mypow(
      f[i], ((double)2) - ((double)1))) * (
      df[i][2]))) * (b[0]), mypow(b[0], 
      (double)2)));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][2]= res;
    res= (df[i][3]) + (mydiv((((double)2) * ((mypow(
      f[i], ((double)2) - ((double)1))) * (
      df[i][3]))) * (b[0]), mypow(b[0], 
      (double)2)));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][3]= res;
    res= (df[i][4]) + (mydiv((((double)2) * ((mypow(
      f[i], ((double)2) - ((double)1))) * (
      df[i][4]))) * (b[0]), mypow(b[0], 
      (double)2)));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][4]= res;
    res= mydiv(-(mypow(f[i], (double)2)), mypow(
      b[0], (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDB;
        return(ERRMATH);
      }
    dbv[i][0]= res;
  }
return(OK);
}

calcodes_()
{}

calcjac_()
{}

calcphi_()
{}
