/* ------------------------------------------------------
Example "ej" of call to nls2C 
The program that calculates the model is in Fortran
A call to "initcommun" initializes the "commun"
declared in it.
 ----------------------------------------------------------- */

#define precision 12 /* Number of significant digits on the written files */

/* +++++++++++++++++++++++++++++++++++++++++++++++++++
 includes 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include "nls2C.h"

/* ------------------------------------------------------------
Calling program
---------------------------------------------------------------*/

int main(argc, argv)
  int argc;
  char *argv[];
{
/* --------  Declarations ------------ */

/* Model programs */
extern void initcommon_(double *, double *, double *, int *); 

/* --------------------------- */

/* Local variables: */
int  code, n1, n2, m1, m2, m3;
double   *theta, *beta;
int i, nparam;


/*	The initial values of the "communs":	 */

 double yinit[10]={
0.057033469999999996, 0.0085287866666666681, 0.0084329133333333334, 
	0.0084027066666666674, 0.0067676066666666661, 0.0060334533333333334, 
	0.0048370066666666668, 0.0044817499999999996, 0.003291213333333333, 
0.002894586666666667};

double xa2 = 66.66666667;
double xh = 5.e-3;
int lgyinit=10;

/* For Splus interface */
int iarg;
char * modelfile;
FILE *filesplus;
char splusfilename[100], dataname[100], objectname[100];
char nominput[100];
TLogic entree= FAUX;
TLogic voulusplus = FAUX;


/* ------- Calling  nls2C ------------ */
if ((argc < 3) || (argc > 5) || (strncmp(argv[1], "-h",2)==0))
  {
  return(erreur());
  }

iarg=1;
while( iarg< argc)
  {

    if (strncmp(argv[iarg], "-s", 2) ==0)
      {
	if (argc <= iarg)
	  {
	   return(erreur());
	  }
	iarg++;
	strcpy(splusfilename, argv[iarg]);
	iarg++;
	voulusplus=VRAI;
	continue;
      }


    if (strncmp(argv[iarg], "-i", 2) ==0)
      {
	if (argc <= iarg)
	  {
	   return(erreur());
	  }
	iarg++;
	strcpy(nominput, argv[iarg]);
	iarg++;
	entree=VRAI;
	continue;
      }

    if (strncmp(argv[iarg], "-h",2) ==0)
      {
	erreur(); /* ce n'est pas une erreur mais c'est pour faire ecrire la syntaxe */
      return(OK);
      }



    if (strncmp(argv[iarg], "-",1) ==0)
      {
	return(erreur());
      }
  } /* fin iarg */

if (entree== FAUX)
  return(erreur()); /* pas d'option -i */



/* Estimation for the three models together: */

/* Call to the program that initializes the Fortran commons: */
initcommon_(yinit, &xh, &xa2, &lgyinit);

if ( (code =nls2C(nominput)) != 0)
  {
  fprintf(stderr,"Fatal error in nls2C, code %d \n", code);
  exit(code);
  }


/* ------- Calling access subroutines --- */
printf(
"\nSome more print to check:\n-------------------------\n");

modelfile = recup1C("file",1);
printf("\nThe name of model-description file: %s\n", modelfile);

/* return the estimated values of the parameters of the last step */
printf("\nValue of the estimated values for 'theta' parameters:\n");
theta=recup1D("theta" , recup0L("n.steps" , 1), &nparam);
for (i=0; i <nparam; i++)
  printf( " %.*g ", precision, theta[i]);
printf( " \n");
nparam = recup0L("n.beta", 1);
if (nparam >0)
  {
    beta=recup1D("beta" , recup0L("n.steps" , 1), &nparam);
    printf("\nValue of the estimated values for 'beta' parameters:\n");
    for (i=0; i <nparam; i++)
      printf( " %.*g ", precision, beta[i]);
    printf( " \n");
  }

/* return of the statistical criterion */
printf("\nValue of the statistical criterion: %g\n", 
       recup0D("stat.crit", 1));

/* return of the type of calculation of the statistical criterion */
printf("\nType of the statistical criterion:\n %s\n",
       recup1C("stat.crit.type",1));



/* Create a splus file for Splus */
if (voulusplus==VRAI)
  {
/* Create a splus file for Splus */
if ( (int)(filesplus=fopen(splusfilename, "w")) == NULL)
  {
    printf("Cannot open the output splus-file %s\n", splusfilename);
    return(ERRINPUT);
  }

sprintf(dataname, "%s.data", modelfile);
sprintf(objectname, "%s.nls2", modelfile);
if ((code= splus2C(objectname ,dataname, filesplus)) != OK)
  {
    printf("Error in 'splus2C'\n");
    return(ERRINPUT);
  }

if ((code= splusData(dataname , filesplus)) != OK)
  {
    printf("Error in 'splusData'\n");
    return(ERRINPUT);
  }

printf("\n\nThe file '%s' is created:\n",splusfilename);
printf("it contains the Splus (or R) command to create an nls2.object\n");
printf("called %s ", objectname); 
printf( " and a data-frame called %s.\n\n", dataname);

  } /* fin voulusplus */

/* Destruction des structures */
delnls2();
return(0);
}



/* ------------------------------------------------------------------
Gestion des erreurs d'appel
-------------------------------------------------------------------- */
int erreur()
{

  fprintf(stderr,"Arguments: [ -s[plus] splusfilename ] -i input-file\n");
  return(ERRARG);
}
