/*----------------------------------------
description file: insect.m13273

%  file insect.m
%  logistic non-linear regression model
%  with binomial distribution
resp y;
var v;
varind DI,DS,n;
aux eta,p;
parresp t1,t2,t3,t4,t5;
subroutine;
begin
eta = t1+t2*log(DI-t3)+t4*DS/(t5+DS);
p=exp(eta)/(1+exp(eta));
y = n*p;
v = n*(1-p)*p;
end
----------------------------------------*/
#include <stdio.h>
#include <math.h>
#include "nltypes.h"
#include "nlcodes.h"
#include "nlmacros.h"
#include "nlfonc.h"

short int calcf_(nbt, nbg, nbl, nbc, t, g, x, f, df, le, ie)
      short int nbt, nbg, nbl, nbc;
      double *t, *g, **x, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
double a[2], da[2][5];
ECRTRACE("calcf");
for (i=0; i<nbl; i++)
  {
    res= ((t[0]) + ((t[1]) * (myln((x[i][0]) - (t[2]))))) + (
      mydiv((t[3]) * (x[i][1]), (t[4]) + (x[i][1])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[0]= res;
    res= mydiv((double)myexp(a[0]), ((double)1) + (
      (double)myexp(a[0])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[1]= res;
    res= (x[i][2]) * (a[1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=F;
        return(ERRMATH);
      }
    f[i]= res;
    res= (double)1;
    da[0][0]= res;
    res= myln((x[i][0]) - (t[2]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][1]= res;
    res= (t[1]) * (mydiv(-((double)1), (x[i][0]) - (
      t[2])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][2]= res;
    res= mydiv((x[i][1]) * ((t[4]) + (x[i][1])), mypow((
      t[4]) + (x[i][1]), (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][3]= res;
    res= mydiv(-((t[3]) * (x[i][1])), mypow((t[4]) + (x[i][1]), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][4]= res;
    res= mydiv((((da[0][0]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][0]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][0]= res;
    res= mydiv((((da[0][1]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][1]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][1]= res;
    res= mydiv((((da[0][2]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][2]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][2]= res;
    res= mydiv((((da[0][3]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][3]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][3]= res;
    res= mydiv((((da[0][4]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][4]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][4]= res;
    res= (x[i][2]) * (da[1][0]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][0]= res;
    res= (x[i][2]) * (da[1][1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][1]= res;
    res= (x[i][2]) * (da[1][2]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][2]= res;
    res= (x[i][2]) * (da[1][3]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][3]= res;
    res= (x[i][2]) * (da[1][4]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][4]= res;
  }
return(OK);
}

short int calcv_(nbt, nbb, nbg, nbl, nbc, t, b, g,
                f, df, x, v, dtv, dbv, le, ie)
      short int nbt, nbb, nbg, nbl, nbc;
      double *t, *b, *g, **x, *v, **dtv, **dbv, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
double a[2], da[2][5];
ECRTRACE("calcv");
for (i=0; i<nbl; i++)
  {
    res= ((t[0]) + ((t[1]) * (myln((x[i][0]) - (t[2]))))) + (
      mydiv((t[3]) * (x[i][1]), (t[4]) + (x[i][1])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[0]= res;
    res= mydiv((double)myexp(a[0]), ((double)1) + (
      (double)myexp(a[0])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[1]= res;
    res= ((x[i][2]) * (((double)1) - (a[1]))) * (
      a[1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=V;
        return(ERRMATH);
      }
    v[i]= res;
    res= (double)1;
    da[0][0]= res;
    res= myln((x[i][0]) - (t[2]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][1]= res;
    res= (t[1]) * (mydiv(-((double)1), (x[i][0]) - (
      t[2])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][2]= res;
    res= mydiv((x[i][1]) * ((t[4]) + (x[i][1])), mypow((
      t[4]) + (x[i][1]), (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][3]= res;
    res= mydiv(-((t[3]) * (x[i][1])), mypow((t[4]) + (x[i][1]), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][4]= res;
    res= mydiv((((da[0][0]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][0]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][0]= res;
    res= mydiv((((da[0][1]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][1]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][1]= res;
    res= mydiv((((da[0][2]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][2]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][2]= res;
    res= mydiv((((da[0][3]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][3]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][3]= res;
    res= mydiv((((da[0][4]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][4]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][4]= res;
    res= (((x[i][2]) * (((double)1) - (a[1]))) * (
      da[1][0])) + (((x[i][2]) * (-(da[1][0]))) * (a[1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][0]= res;
    res= (((x[i][2]) * (((double)1) - (a[1]))) * (
      da[1][1])) + (((x[i][2]) * (-(da[1][1]))) * (a[1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][1]= res;
    res= (((x[i][2]) * (((double)1) - (a[1]))) * (
      da[1][2])) + (((x[i][2]) * (-(da[1][2]))) * (a[1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][2]= res;
    res= (((x[i][2]) * (((double)1) - (a[1]))) * (
      da[1][3])) + (((x[i][2]) * (-(da[1][3]))) * (a[1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][3]= res;
    res= (((x[i][2]) * (((double)1) - (a[1]))) * (
      da[1][4])) + (((x[i][2]) * (-(da[1][4]))) * (a[1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][4]= res;
  }
return(OK);
}

calcodes_()
{}

calcjac_()
{}

calcphi_()
{}
