/* ------------------------------------------------------
Example "mag" of call to renls2C 
   Subroutines of the Nag library are used to generate pseudo-observations.

 ----------------------------------------------------------- */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++
 includes 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include "nls2C.h"

double * oneloop (int b, double *p, double *f,
		  long *Bool, long *VectBool, 
              long Nb, double *newresp,
		  double *theta1, double *resi1)
{
  /* ---------------------------------------------------------------
 This function realizes the instructions of one loop 
 Arguments:
 b: loop index ( used for printing messages)
 p: bernoulli probabability for the random generator
 f: to store the new values of the response
 Bool, VectBool, Nb: arguments of the random generator
 VectBool is a vector of dim Nb=n
 Nb: dimension of the response, dim=n
 newresp: new values of the response
 theta1,resi1: values of the theta and residuals at the 1st estimation
 Result:
  the result of the estimation of theta
  --------------------------------------------------------------- */

/*------------- Declarations  --------------------------------  */
int i, n1, Code=OK;

/* ---- external functions --------------------------------------*/
  /* Random generation of a n-vector containing 0-1 */
extern void tirage(double *p, long *Bool, long *VectBool,long *Lg);

/* ------- Instructions ---------------------------- */
tirage( p, Bool, VectBool, &Nb);

/* The 0-1 vector is transformed to facilitate the expression
   that generates the new values of f: */
for (i=0; i< Nb; i++)
  {
    if (VectBool[i]==1) VectBool[i]= -1;
    if (VectBool[i]==0) VectBool[i]= 1;
  }

/* Creation of the new values of f from the preceeding estimated
 values, the residuals calculated at the first estimation
 and the random vector: */
/* Replicate the k estimated values of the response => dim=n */
rep2C( newresp, f);
for (i=0; i< Nb; i++)
  {
  f[i]=f[i] +
        (( 1  + (VectBool[i]*sqrt(5)) /2 ) * resi1[i]);
  }


/* The new estimation is realized by call to the function "renls2C": */

Code=renls2C(theta1, (double *)NULL, 
	    (double *)NULL, (double *)NULL,
            (double **)NULL,
	    f, (double *)NULL, 0);

if (Code != (int)OK)
  {
    fprintf(stderr,
"Wrong execution of the loop %d code = %d\n",
b, Code);
    exit(Code);
  }

/* Printing of some of the results: */
printf("\nNumber of iterations at the loop %d: %d\n",
       (b+1), recup0L("n.iters",1));

return(recup1D("theta",1, &n1));

} 
/* ----- end of the function oneloop -------------------------*/

/* ------------------------------------------------------------
Calling program
---------------------------------------------------------------*/

int main(argc, argv)
  int argc;
  char *argv[];
{
/* --------  Declarations ------------ */


/* Local variables: */
double p;
long *VectBool, Nb, Bool;
int iarg, n1, ntheta, code=OK,  b, i;
double *newresp,  *theta, *resi1, *theta1, *f;
/* For Splus interface */
char * modelfile;
FILE *filesplus;
char splusfilename[100], dataname[100], objectname[100];
char nominput[100];
TLogic voulusplus = FAUX;
TLogic entree= FAUX;

/* Argument treatement -----------------------------*/
if ((argc < 3) || (argc > 5) || (strncmp(argv[1], "-h",2)==0))
  {
  return(erreur());
  }

iarg=1;
while( iarg< argc)
  {

    if (strncmp(argv[iarg], "-s", 2) ==0)
      {
	if (argc <= iarg)
	  {
	   return(erreur());
	  }
	iarg++;
	strcpy(splusfilename, argv[iarg]);
	iarg++;
	voulusplus=VRAI;
	continue;
      }


    if (strncmp(argv[iarg], "-i", 2) ==0)
      {
	if (argc <= iarg)
	  {
	   return(erreur());
	  }
	iarg++;
	strcpy(nominput, argv[iarg]);
	iarg++;
	entree=VRAI;
	continue;
      }

    if (strncmp(argv[iarg], "-h",2) ==0)
      {
	erreur(); /* ce n'est pas une erreur mais c'est pour faire ecrire la syntaxe */
      return(OK);
      }



    if (strncmp(argv[iarg], "-",1) ==0)
      {
	return(erreur());
      }
  } /* fin iarg */

if (entree== FAUX)
  return(erreur()); /* pas d'option -i */


/* ------- Calling  nls2C ------------ */

/* First invokation of nls2C: */

if ( (code =nls2C(nominput)) != 0)
  {
  fprintf(stderr,"Fatal error in nls2C, code %d \n", code);
  exit(code);
  }

/* We keep the first results */
resi1=recup1D("residuals", 1, &n1);
Nb=(long)n1;
theta1=recup1D("theta", 1, &ntheta);


/* Initialization of the random seed: call to Nag function */
/* p= bernoulli probabability for the random generator */
p= (5. + sqrt(5))/ 10.0 ;

/* initialization of the random seed by call to the Nag function "g05cbf": */
Bool = 0;
g05cbf_(&Bool);

/* initialization of the vector containing the random numbers: */
VectBool  = (long *) calloc (Nb, sizeof (long));

/* allocate of f: working vector to store the new values of the response */
f=(double *) calloc (Nb, sizeof (double));

for (b=0; b <3; b++)
  {
    /* At each loop, the new values of the response are generated from
       the values estimated at the preceeding loop */
    newresp =recup1D("fitted",1, &n1);
    theta= oneloop ( b, &p, f, &Bool, VectBool, 
              Nb, newresp, theta1, resi1);
    /* print */
    printf("Estimated values of 'theta' at loop %d:\n", (b+1));
    for (i=0; i< ntheta; i++)
      printf("%g ", theta[i]);
    printf("\n\n");
  }


/* Create a splus file for Splus */
if (voulusplus==VRAI)
  {
if ( (int)(filesplus=fopen(splusfilename, "w")) == NULL)
  {
    printf("Cannot open the output splus-file %s\n", splusfilename);
    return(ERRINPUT);
  }

modelfile = recup1C("file",1);

if ( (int)(filesplus=fopen(splusfilename, "w")) == NULL)
  {
    printf("Cannot open the output splus-file %s\n", splusfilename);
    return(ERRINPUT);
  }

sprintf(dataname, "%s.data", modelfile);
sprintf(objectname, "%s.nls2", modelfile);
if ((code= splus2C(objectname , dataname, filesplus)) != OK)
  {
    printf("Error in 'splus2C'\n");
    return(ERRINPUT);
  }

if ((code= splusData(dataname , filesplus)) != OK)
  {
    printf("Error in 'splusData'\n");
    return(ERRINPUT);
  }


printf("\n\nThe file '%s' is created:\n",splusfilename);
printf("it contains the Splus (or R) command to create an nls2.object\n");
printf("called %s ", objectname); 
printf( " and a data-frame called %s.\n\n", dataname);

  } /* fin voulusplus */

delnls2();

return(0);
}


/* ------------------------------------------------------------------
Gestion des erreurs d'appel
-------------------------------------------------------------------- */
int erreur()
{

  fprintf(stderr,"Arguments: [ -s[plus] splusfilename ] -i input-file\n");
  return(ERRARG);
}
