/*----------------------------------------
description file: orobranche.m513415

% file orobranche.m5
% logistic linear regression model
% overdispersion : Gaussian random effect on eta
resp y; 
varind sp,ex,n;
aux eta,p; 
var v;
parresp a,b,c,d;
parvar tau;
subroutine;
begin 
eta=a + b*sp + c*ex + d*sp*ex;
p=exp(eta)/(1+exp(eta));
y = n*p;
v = n*(1-p)*p*(1+(n-1)*tau*p*(1-p));
end 
----------------------------------------*/
#include <stdio.h>
#include <math.h>
#include "nltypes.h"
#include "nlcodes.h"
#include "nlmacros.h"
#include "nlfonc.h"

short int calcf_(nbt, nbg, nbl, nbc, t, g, x, f, df, le, ie)
      short int nbt, nbg, nbl, nbc;
      double *t, *g, **x, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
double a[2], da[2][4];
ECRTRACE("calcf");
for (i=0; i<nbl; i++)
  {
    res= (((t[0]) + ((t[1]) * (x[i][0]))) + ((t[2]) * (x[i][1]))) + (
      ((t[3]) * (x[i][0])) * (x[i][1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[0]= res;
    res= mydiv((double)myexp(a[0]), ((double)1) + (
      (double)myexp(a[0])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[1]= res;
    res= (x[i][2]) * (a[1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=F;
        return(ERRMATH);
      }
    f[i]= res;
    res= (double)1;
    da[0][0]= res;
    res= x[i][0];
    da[0][1]= res;
    res= x[i][1];
    da[0][2]= res;
    res= (x[i][0]) * (x[i][1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][3]= res;
    res= mydiv((((da[0][0]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][0]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][0]= res;
    res= mydiv((((da[0][1]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][1]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][1]= res;
    res= mydiv((((da[0][2]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][2]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][2]= res;
    res= mydiv((((da[0][3]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][3]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][3]= res;
    res= (x[i][2]) * (da[1][0]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][0]= res;
    res= (x[i][2]) * (da[1][1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][1]= res;
    res= (x[i][2]) * (da[1][2]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][2]= res;
    res= (x[i][2]) * (da[1][3]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DFDT;
        return(ERRMATH);
      }
    df[i][3]= res;
  }
return(OK);
}

short int calcv_(nbt, nbb, nbg, nbl, nbc, t, b, g,
                f, df, x, v, dtv, dbv, le, ie)
      short int nbt, nbb, nbg, nbl, nbc;
      double *t, *b, *g, **x, *v, **dtv, **dbv, *f, **df;
      short int *le, *ie;
{
short int i;
double res;
double a[2], da[2][5];
ECRTRACE("calcv");
for (i=0; i<nbl; i++)
  {
    res= (((t[0]) + ((t[1]) * (x[i][0]))) + ((t[2]) * (x[i][1]))) + (
      ((t[3]) * (x[i][0])) * (x[i][1]));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[0]= res;
    res= mydiv((double)myexp(a[0]), ((double)1) + (
      (double)myexp(a[0])));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=A;
        return(ERRMATH);
      }
    a[1]= res;
    res= (((x[i][2]) * (((double)1) - (a[1]))) * (
      a[1])) * (((double)1) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (a[1])) * ((
      (double)1) - (a[1]))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=V;
        return(ERRMATH);
      }
    v[i]= res;
    res= (double)1;
    da[0][0]= res;
    res= x[i][0];
    da[0][1]= res;
    res= x[i][1];
    da[0][2]= res;
    res= (x[i][0]) * (x[i][1]);
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[0][3]= res;
    res= (double)0;
    da[0][4]= res;
    res= mydiv((((da[0][0]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][0]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][0]= res;
    res= mydiv((((da[0][1]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][1]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][1]= res;
    res= mydiv((((da[0][2]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][2]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][2]= res;
    res= mydiv((((da[0][3]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][3]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADT;
        return(ERRMATH);
      }
    da[1][3]= res;
    res= mydiv((((da[0][4]) * ((double)myexp(a[0]))) * ((
      (double)1) + ((double)myexp(a[0])))) - (((double)myexp(
      a[0])) * ((da[0][4]) * ((double)myexp(a[0])))), mypow(
      ((double)1) + ((double)myexp(a[0])), 
      (double)2));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DADB;
        return(ERRMATH);
      }
    da[1][4]= res;
    res= ((((x[i][2]) * (((double)1) - (a[1]))) * (
      a[1])) * ((((((x[i][2]) - ((double)1)) * (
      b[0])) * (a[1])) * (-(da[1][0]))) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (da[1][0])) * ((
      (double)1) - (a[1]))))) + (((((x[i][2]) * (
      ((double)1) - (a[1]))) * (da[1][0])) + ((
      (x[i][2]) * (-(da[1][0]))) * (a[1]))) * ((
      (double)1) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (a[1])) * ((
      (double)1) - (a[1])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][0]= res;
    res= ((((x[i][2]) * (((double)1) - (a[1]))) * (
      a[1])) * ((((((x[i][2]) - ((double)1)) * (
      b[0])) * (a[1])) * (-(da[1][1]))) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (da[1][1])) * ((
      (double)1) - (a[1]))))) + (((((x[i][2]) * (
      ((double)1) - (a[1]))) * (da[1][1])) + ((
      (x[i][2]) * (-(da[1][1]))) * (a[1]))) * ((
      (double)1) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (a[1])) * ((
      (double)1) - (a[1])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][1]= res;
    res= ((((x[i][2]) * (((double)1) - (a[1]))) * (
      a[1])) * ((((((x[i][2]) - ((double)1)) * (
      b[0])) * (a[1])) * (-(da[1][2]))) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (da[1][2])) * ((
      (double)1) - (a[1]))))) + (((((x[i][2]) * (
      ((double)1) - (a[1]))) * (da[1][2])) + ((
      (x[i][2]) * (-(da[1][2]))) * (a[1]))) * ((
      (double)1) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (a[1])) * ((
      (double)1) - (a[1])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][2]= res;
    res= ((((x[i][2]) * (((double)1) - (a[1]))) * (
      a[1])) * ((((((x[i][2]) - ((double)1)) * (
      b[0])) * (a[1])) * (-(da[1][3]))) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (da[1][3])) * ((
      (double)1) - (a[1]))))) + (((((x[i][2]) * (
      ((double)1) - (a[1]))) * (da[1][3])) + ((
      (x[i][2]) * (-(da[1][3]))) * (a[1]))) * ((
      (double)1) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (a[1])) * ((
      (double)1) - (a[1])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDT;
        return(ERRMATH);
      }
    dtv[i][3]= res;
    res= ((((x[i][2]) * (((double)1) - (a[1]))) * (
      a[1])) * ((((((x[i][2]) - ((double)1)) * (
      b[0])) * (a[1])) * (-(da[1][4]))) + ((((((x[i][2]) - (
      (double)1)) * (b[0])) * (da[1][4])) + (((
      x[i][2]) - ((double)1)) * (a[1]))) * ((
      (double)1) - (a[1]))))) + (((((x[i][2]) * (
      ((double)1) - (a[1]))) * (da[1][4])) + ((
      (x[i][2]) * (-(da[1][4]))) * (a[1]))) * ((
      (double)1) + (((((x[i][2]) - (
      (double)1)) * (b[0])) * (a[1])) * ((
      (double)1) - (a[1])))));
    if (finite(res)!=1)
      {
        *ie=i+1;
        *le=DVDB;
        return(ERRMATH);
      }
    dbv[i][0]= res;
  }
return(OK);
}

calcodes_()
{}

calcjac_()
{}

calcphi_()
{}
