/* ------------------------------------------------------
Example "simul" of call to nls2C 
The program that calculates the model is in Fortran
A call to "initcommun" initializes the "commun"
declared in it.
 ----------------------------------------------------------- */
#define precision 12 /* Number of significant digits on the written files */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++
 includes 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include "nls2C.h"

/* ------------------------------------------------------------
Calling program
---------------------------------------------------------------*/

int main(argc, argv)
  int argc;
  char *argv[];
{
/* --------  Declarations ------------ */

/* Model programs */
extern void initcommon_(double *, double *, double *, double *, 
			double *, double *, double *, double *, 
			double *, double *, double *,double *,
			int *,double *, double *);

/* --------------------------- */

/* problem constants: */
double xa = 66.67, xr = 0.086, cl = 26587, cns = 1100;
double xk = 1.3, xka = 3.3, xk1 = 1.37, coefct = 11.95;
double h = 0.5e-3,  eps = 1.0e-6,  eta = 0.0,  hbase = 0.00005;

/* Local variables: */
int  iarg, i, nparam, code, n1, n2, m1, m2, m3;
double *theta, *beta;

/* For Splus interface */
char * modelfile;
FILE *filesplus;
char splusfilename[100], dataname[100], objectname[100];
char nominput[100];
TLogic voulusplus = FAUX;
TLogic entree= FAUX;

/* ------- Calling  nls2C ------------ */
if ((argc < 3) || (argc > 5) || (strncmp(argv[1], "-h",2)==0))
  {
  return(erreur());
  }

iarg=1;
while( iarg< argc)
  {

    if (strncmp(argv[iarg], "-s", 2) ==0)
      {
	if (argc <= iarg)
	  {
	   return(erreur());
	  }
	iarg++;
	strcpy(splusfilename, argv[iarg]);
	iarg++;
	voulusplus=VRAI;
	continue;
      }


    if (strncmp(argv[iarg], "-i", 2) ==0)
      {
	if (argc <= iarg)
	  {
	   return(erreur());
	  }
	iarg++;
	strcpy(nominput, argv[iarg]);
	iarg++;
	entree=VRAI;
	continue;
      }

    if (strncmp(argv[iarg], "-h",2) ==0)
      {
	erreur(); /* ce n'est pas une erreur mais c'est pour faire ecrire la syntaxe */
      return(OK);
      }



    if (strncmp(argv[iarg], "-",1) ==0)
      {
	return(erreur());
      }
  } /* fin iarg */

if (entree== FAUX)
  return(erreur()); /* pas d'option -i */


	
/* nls2C initialisation: affect the default values */
if ((code=  (int)init2C()) !=(int)OK)
  {
    ecrErr(stderr, "mainC", "Error initialisation\n", code);
  return(code);
  }

/* Read the input files */
if ( (code=(int)lect2C(nominput) ) !=(int)OK)
  {
    ecrErr(stderr, "nls2C", "Error in lecture.\n", code);
  return(code);
  }

n1=recup0L("n.obst", 1);

/* Call to the program that initializes the Fortran commons: */
initcommon_( 
       &xa, &xr, &cl, 
       &cns, &xk, &xka, 
       &xk1, &coefct, 
       &h, & eps, & eta, & hbase, 
       &n1,
       recup2D("x",1, &m1, &m2)[0],
       recup1D("y",1,&m1));



/* ------- Calling  nls2C ------------ */

if ( (code =go2C()) != 0)
  {
  fprintf(stderr,"Fatal error in go2C, code %d \n", code);
  exit(code);
  }

/* ------- Calling access subroutines --- */
printf(
"\nSome more print to check:\n-------------------------\n");

modelfile = recup1C("file",1);
printf("\nThe name of model-description file: %s\n", modelfile);

/* return the estimated values of the parameters of the last step */
printf("\nValue of the estimated values for 'theta' parameters:\n");
theta=recup1D("theta" , recup0L("n.steps" , 1), &nparam);
for (i=0; i <nparam; i++)
  printf( " %.*g ", precision, theta[i]);
printf( " \n");
nparam = recup0L("n.beta", 1);
if (nparam >0)
  {
    beta=recup1D("beta" , recup0L("n.steps" , 1), &nparam);
    printf("\nValue of the estimated values for 'beta' parameters:\n");
    for (i=0; i <nparam; i++)
      printf( " %.*g ", precision, beta[i]);
    printf( " \n");
  }

/* return of the statistical criterion */
printf("\nValue of the statistical criterion: %g\n", 
       recup0D("stat.crit", 1));

/* return of the type of calculation of the statistical criterion */
printf("\nType of the statistical criterion:\n %s\n",
       recup1C("stat.crit.type",1));



/* Create a splus file for Splus */
if (voulusplus==VRAI)
  {
if ( (int)(filesplus=fopen(splusfilename, "w")) == NULL)
  {
    printf("Cannot open the output splus-file %s\n", splusfilename);
    return(ERRINPUT);
  }

sprintf(dataname, "%s.data", modelfile);
sprintf(objectname, "%s.nls2", modelfile);
if ((code= splus2C(objectname , dataname, filesplus)) != OK)
  {
    printf("Error in 'splus2C'\n");
    return(ERRINPUT);
  }

if ((code= splusData(dataname , filesplus)) != OK)
  {
    printf("Error in 'splusData'\n");
    return(ERRINPUT);
  }


printf("\n\nThe file '%s' is created:\n",splusfilename);
printf("it contains the Splus (or R) command to create an nls2.object\n");
printf("called %s ", objectname); 
printf( " and a data-frame called %s.\n\n", dataname);

  } /* fin voulusplus */

/* Destruction des structures */
delnls2();
return(0);
}



/* ------------------------------------------------------------------
Gestion des erreurs d'appel
-------------------------------------------------------------------- */
int erreur()
{

  fprintf(stderr,"Arguments: [ -s[plus] splusfilename ] -i input-file\n");
  return(ERRARG);
}
