#ifndef NLS2CH
#define NLS2CH

/* +++++++++++++++++++++++++++++++++++++++++++++++++++
les define
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* Les codes d'erreur de NL etant positifs,
   on met des codes negatifs pour le reste */
#define ERRINPUT -1 /* erreur de lecture */
#define ERRCALLOC -2 /* erreur d'alloc */
#define ERRARG -3 /* erreur d'argument */
#define ERRRECUP -4 /* erreur dans les valeurs recuperees */
#define FINOF -5 /* fin de fichier rencontre dans les entres */
#define ERRFOUND -6 /* une structure  rcuprer n'a pas t calcule */
#define ERRINT -10 /* internal erreur=Erreur dans mes programmes */

/* Warnings */
#define WARNULL -20 /* Toutes les valeurs d'une structure  rcuprer sont nulles */

#define MAXVA 10 
/*maximum number of independent variables that are integration variables */
/* cf initCtxI */
#define LGALLOC 10 /* A la lecture, on alloue les vecteurs par tranche */

/* Pour la lecture: cf lecture.c et litFile.c */
#define MAXLIG 655
#define MAXMOT 50

/* +++++++++++++++++++++++++++++++++++++++++++++++++++
les includes 
++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* includes C */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

/* pour nl */
#include "nltypes.h"
#include "nlcodes.h"
#include "errcodes.h"
#include "nlmacros.h"
#include "nlchoix.h"

/* l'appel  NL: */
#include "nldcl.h"  

/* pour l'appel de analder */
#include "dftypes.h" 
/* pour l'appel de memwatch */
#ifdef MEMWATCH
#include "memwatch.h"
#endif
/*--------------- LES FONCTIONS SYSTEMES -------- */

extern char *strcpy(char *s1, const char *s2);
extern char *strtok(char *s1, const char *s2);
extern int strncmp(const char *s1, const char *s2, size_t n);
extern int strcmp(const char *s1, const char *s2);


/*--------------- VARIABLES GLOBALES -------------*/
/* Les variables globales de nl */
#include "nlglobal.h"
/*  contraintes sur les parametres differents. */
TContr CThetaE[MAXETAP], CBetaE[MAXETAP];



/* +++++++++++++++++++++++++++++++++++++++++++++++++++
 Declarations of the nls2C subroutines: 
 +++++++++++++++++++++++++++++++++++++++++++++++++++ */
  /* To execute */
extern int nls2C(char *filein);
extern int init2C();
extern int lect2C(char *filein);
extern int go2C();
extern int renls2C(double *ThetaInit, double *BetaInit, 
	    double *GamF, double *GamV,
            double **x,
	    double *y,
	    double *poids,
		   int ip);

/* To set in dimension n a struct in dimension k */
extern void rep2C(double * datak, double *f);

/* To access results */
extern long   recup0L(char *keyword ,int step);
extern double   recup0D(char *keyword ,int step);
extern char *  recup1C(char *nom ,int step);
extern double * recup1D(char *keyword ,int step, int *n1);
extern double **recup2D(char *keyword ,int step,int *n1, int *n2);
extern double ***recup3D(char *nom ,int step, 
			 int *nb1, int *nb2, int *nb3);

/* To access intermediate results */
extern long int * recupIt1L(char *name, int step,int *nitsv);
extern double * recupIt1D(char *name, int step,int *nitsv);
extern double ** recupIt2D(char *name, int step, int *nitsv, int *n1) ;
extern double *** recupIt3D(char *name, int step, int *nitsv, int *n1, int *n2);
extern double **** recupIt4D(char *name, int step, int *nitsv, 
     int *n1, int *n2, int *n3);

/* To print input */
extern void ecrDonnees(FILE *fic);
extern void ecrModele(FILE *fic);
extern void ecrCtxPuss(FILE *fic);
extern void ecrCtxNum(FILE *fic);
extern void ecrCrole(FILE *fic);
extern void ecrCtxI(FILE *fic);

/* To print results */
extern void pr1L(FILE *unit, long *vect, int nele);
extern void pr1D(FILE *unit, double *vect, int nele);
extern void pr1S(FILE *unit, short *vect, int nele);
extern void pr2S(FILE *unit, short **mat, int nlig, int ncol);
extern void pr2D(FILE *unit, double **mat, int nlig, int ncol);
extern void pr3D(FILE *unit, double ***mat, int nlig, int ncol, int n3);

/* To create files for Splus */
extern int splus2C(char *nom , char *nomdata, FILE *fic);
extern int splusData( char *nom , FILE *fic);

/* Program in nls2/src */
extern void delnls2();

/* +++++++++++++++++++++++++++++++++++++++++++++++++++
les macros
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CREER_T3(tab,n0,type) \
  { \
  tab = (type ***) calloc (n0, sizeof (type **)); \
  if (tab == NULL) \
    { \
    fprintf(stdout, "Memory allocation problem\n"); \
    exit(ERRALLOC); \
    } \
  }


#define CREER_T4(tab,n0,type) \
  { \
  tab = (type ****) calloc (n0, sizeof (type ***)); \
  if (tab == NULL) \
    { \
    fprintf(stdout, "Memory allocation problem\n"); \
    exit(ERRALLOC); \
    } \
  }
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#endif
