
/* +++++++++++++++++++++++++++++++++++++++++++++++++++
les define et les includes 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <ctype.h>
#include "nls2C.h"

/* pour crire les messages d'erreur */
extern void ecrErr(FILE *fic, char * nomcall , char *message, int Code);

/* ----------------------------------------------------------------------
Recuperation des valeurs stockes dans les structures
(toutes sont dcl en global) de nls2
------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------
Teste si les 'donnees' d'une matrice ne contient que des zeros.
Si oui, renvoie NULL, sinon renvoie celle-ci
------------------------------------------------------------------------- */
double **test2D(char * nom, double **donnees, int nblig, int nbcol)
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];

int i,j;
for (i=0; i<nblig; i++)
  {
    for (j=0; j< nbcol; j++)
      {
	if (donnees[i][j] != (double)0)
	  return(donnees);
      }
  }
sprintf(message, "No %s: all values are null  \n", nom);
ecrErr(stderr, "test2D", message, WARNULL);
/* fprintf(stdout,"No %s: all values are null  \n", nom);*/
return((double **)NULL);

  }
/* --------------- fin test2D ----------------------------------*/


/* ----------------------------------------------------------------------
Teste si les 'donnees' d'un tableau a 3 dim ne contient que des zeros.
Si oui, renvoie NULL, sinon renvoie celui-ci
------------------------------------------------------------------------- */
double ***test3D(char * nom,
		    double ***donnees, int nblig, int nbcol, int nb3)
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i,j,k;
for (i=0; i<nblig; i++)
  {
    for (j=0; j< nbcol; j++)
      {
	for (k=0; k< nb3; k++)
	  {
	    if (donnees[i][j][k] != (double)0)
	      return(donnees);
	  }
      }
  }
sprintf(message, "No %s: all values are null  \n", nom);
ecrErr(stderr, "test3D", message, WARNULL);
return((double ***)NULL);
  }
/* --------------- fin test3D ----------------------------------*/


/* ----------------------------------------------------------------------
Complete  as.var d'une etape >1 (donneess).
Les donnees manquantes sont remplaces par les valeurs correspondantes
calcules  l'tape prcdente (donneesp)
------------------------------------------------------------------------- */
double **testAsVar(double **donneess, double **donneesp,
         int nblig, int nbcol)
{
int i,j;
for (i=0; i<nblig; i++)
  {
    for (j=0; j< nbcol; j++)
      {
	if (donneess[i][j] == DBL_MAX)
	  donneess[i][j] = donneesp[i][j];
      }
  }

return(donneess);

  }
/* --------------- fin testAsVar ----------------------------------*/

/* ----------------------------------------------------------------------
Recuperation d'une matrice double
------------------------------------------------------------------------- */
double **recup2D(char *argunom ,int step,int *nblig, int *nbcol)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nblig, nbcol : le nbre d'elements */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i;
double **retour= (double **)NULL;

char nom[MAXLGMESS];

strcpy(nom, argunom);

*nblig= *nbcol=0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recup2D", message, ERRARG);
    /*
    fprintf(stdout,"Error in recup2D: step %d should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    */
    return(retour);
  }
 step--; /* mise a partir de zero */

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);


if (strncmp(nom, "as.var", 4)==0)
  {
    if ( (ResStat[step].AsVar.donnees==NULL) ||
	 (ResStat[step].AsVar.nblig <=0) ||
	 (ResStat[step].AsVar.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].AsVar.nblig;
    *nbcol=ResStat[step].AsVar.nbcol;
    if (step >1)
      {
	/* On complete les donnees d'une as.var d'une etape >1 
par les donnees calcules  l'tape prcdente  pour l'autre jeu de
parametres (soit theta, soit beta) */
	ResStat[step].AsVar.donnees= testAsVar(
ResStat[step].AsVar.donnees,ResStat[step-1].AsVar.donnees,
*nblig, *nbcol);
      }
    return(test2D(nom, ResStat[step].AsVar.donnees, *nblig, *nbcol));
  }


if (strncmp(nom, "d.resp", 3)==0)
  {
    if ( (ResStat[step].FctSensib.DValf.donnees==NULL) ||
	 (ResStat[step].FctSensib.DValf.nblig <=0) ||
	 (ResStat[step].FctSensib.DValf.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].FctSensib.DValf.nblig;
    *nbcol=ResStat[step].FctSensib.DValf.nbcol;
    return(test2D(nom, ResStat[step].FctSensib.DValf.donnees, *nblig, *nbcol));
  }



if (strncmp(nom, "d.theta.vari", 3)==0)
  {
    if ( (ResStat[step].FctSensib.DVarYTheta.donnees==NULL) ||
	 (ResStat[step].FctSensib.DVarYTheta.nblig <=0) ||
	 (ResStat[step].FctSensib.DVarYTheta.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].FctSensib.DVarYTheta.nblig;
    *nbcol=ResStat[step].FctSensib.DVarYTheta.nbcol;
    return(test2D(nom, ResStat[step].FctSensib.DVarYTheta.donnees, *nblig, *nbcol));
  }


if (strncmp(nom, "d.beta.vari", 3)==0)
  {
    if ( (ResStat[step].FctSensib.DVarYBeta.donnees==NULL) ||
	 (ResStat[step].FctSensib.DVarYBeta.nblig <=0) ||
	 (ResStat[step].FctSensib.DVarYBeta.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].FctSensib.DVarYBeta.nblig;
    *nbcol=ResStat[step].FctSensib.DVarYBeta.nbcol;
    return(test2D(nom, ResStat[step].FctSensib.DVarYBeta.donnees, *nblig, *nbcol));
  }


if (strncmp(nom, "b.varz.b", 4)==0)
  {
    if ( (ResStat[step].BVarZBP.donnees==NULL) ||
	 (ResStat[step].BVarZBP.nblig <=0) ||
	 (ResStat[step].BVarZBP.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].BVarZBP.nblig;
    *nbcol=ResStat[step].BVarZBP.nbcol;
    return(test2D(nom, ResStat[step].BVarZBP.donnees, *nblig, *nbcol));
  }

if (strncmp(nom, "correlation", 3)==0)
  {
    if ( (ResStat[step].Corr.donnees==NULL) ||
	 (ResStat[step].Corr.nblig <=0) ||
	 (ResStat[step].Corr.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].Corr.nblig;
    *nbcol=ResStat[step].Corr.nbcol;
    return(test2D(nom, ResStat[step].Corr.donnees, *nblig, *nbcol));
  }


/* ATTENTION: tester a la fin les noms qui commencent
   idem que qqche d'autres mais qui sont plus courts */

if (strncmp(nom, "w", 1)==0)
  {
    if ( (ResStat[step].ValW.donnees==NULL) ||
	 (ResStat[step].ValW.nblig <=0) ||
	 (ResStat[step].ValW.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].ValW.nblig;
    *nbcol=ResStat[step].ValW.nbcol;
    return(test2D(nom, ResStat[step].ValW.donnees, *nblig, *nbcol));
  }

if (strncmp(nom, "b", 1)==0)
  {

    if ( (ResStat[step].EquN.ValB.donnees==NULL) ||
	 (ResStat[step].EquN.ValB.nblig <=0) ||
	 (ResStat[step].EquN.ValB.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].EquN.ValB.nblig;
    *nbcol=ResStat[step].EquN.ValB.nbcol;
    return(test2D(nom, ResStat[step].EquN.ValB.donnees, *nblig, *nbcol));
  }


if (strncmp(nom, "d", 1)==0)
  {
    if ( (ResStat[step].EquN.ValD.donnees==NULL) ||
	 (ResStat[step].EquN.ValD.nblig <=0) ||
	 (ResStat[step].EquN.ValD.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=ResStat[step].EquN.ValD.nblig;
    *nbcol=ResStat[step].EquN.ValD.nbcol;
    return(test2D(nom, ResStat[step].EquN.ValD.donnees, *nblig, *nbcol));
  }


/* Les donnees */

if (strncmp(nom, "x", 1)==0)
  {
    if ( (Donnees.XObsT.donnees==NULL) ||
	 (Donnees.XObsT.nblig <=0) ||
	 (Donnees.XObsT.nbcol <=0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup2D", message, ERRFOUND);
	return(retour);
      }
    *nblig=Donnees.XObsT.nblig;
    *nbcol=Donnees.XObsT.nbcol;
    return(test2D(nom, Donnees.XObsT.donnees, *nblig, *nbcol));
  }

sprintf(message,"%s not valid!\n",nom);
ecrErr(stderr, "recup2D", message, ERRARG);

return(retour);

}
/* fin recup2D */


/* ----------------------------------------------------------------------
Recuperation d'une chaine de car
------------------------------------------------------------------------- */
char *  recup1C(char *argunom ,int step)

{
/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* Pour les messages d'erreur */
char message[MAXLGMESS];

int i,k;
char * retour = NULL ;
int nbmess=15, nbalgo=2, nbcrit=11, nbmeth=14, nbsym=3, nbmu=4,
   nbsigma=5, nbfamille=5, nbvari=8;
 char * TexteMu[4];
char * TexteAlgo[2];
char * TexteSym[3];
 char * TexteMeth[14];
 char * TexteSigma[5];
 char * TexteCrit[11], * TexteCritCode[11];
 char *TexteMsg[15],  *TexteFamille[5];
char *TexteVari[8];

int CodeMsg[15];


char nom[MAXLGMESS];

strcpy(nom, argunom);

for (i=0; i< nbmu; i++)
  TexteMu[i] = (char *) calloc( MAXLGMESS, sizeof (char));

for (i=0; i< nbvari; i++)
  TexteVari[i] = (char *) calloc( MAXLGMESS, sizeof (char));


for (i=0; i< nbmess; i++)
  TexteMsg[i] = (char *) calloc( MAXLGMESS, sizeof (char));

for (i=0; i< nbalgo; i++)
  TexteAlgo[i] = (char *) calloc( MAXLGMESS, sizeof (char));

 for (i=0; i< nbcrit; i++) {
  TexteCrit[i] = (char *) calloc( MAXLGMESS, sizeof (char));
  TexteCritCode[i] = (char *) calloc( MAXLGMESS, sizeof (char));
 }

for (i=0; i< nbmeth; i++)
  TexteMeth[i] = (char *) calloc( MAXLGMESS, sizeof (char));

for (i=0; i< nbsym; i++)
  TexteSym[i] = (char *) calloc( MAXLGMESS, sizeof (char));

for (i=0; i< nbsigma; i++)
  TexteSigma[i] = (char *) calloc( MAXLGMESS, sizeof (char));

 for (i=0; i< nbfamille; i++)
  TexteFamille[i] = (char *) calloc( MAXLGMESS, sizeof (char));

i=0;
strcpy(TexteMu[i++], "KNOWN");
strcpy(TexteMu[i++],"MUGAUSS");
strcpy(TexteMu[i++],"MURES"); 
strcpy(TexteMu[i++],"MURESREP");

i=0;
strcpy(TexteSym[i++], "SYM"); 
strcpy(TexteSym[i++],"SYMBLOCK"); 
strcpy(TexteSym[i++], "NONSYM"); 
i=0;
strcpy(TexteAlgo[i++],"GN");  strcpy(TexteAlgo[i++],"GM"); 

i=0;
strcpy(TexteMeth[i++],  "MLTB"); 
strcpy(TexteMeth[i++],"MLSTB"); 
strcpy(TexteMeth[i++],"ERR"); 
strcpy(TexteMeth[i++],"MLT"); 
strcpy(TexteMeth[i++],"WLST"); 
strcpy(TexteMeth[i++],"OLST"); 
strcpy(TexteMeth[i++],"MLST"); 
strcpy(TexteMeth[i++],"VITWLS"); 
strcpy(TexteMeth[i++],"OLSB"); 
strcpy(TexteMeth[i++],"MLSB"); 
 strcpy(TexteMeth[i++],"QLTB"); 
 strcpy(TexteMeth[i++],"QLT");
 strcpy(TexteMeth[i++],"QLB");
strcpy(TexteMeth[i++],"MYOWN"); 

i=0;
strcpy(TexteSigma[i++],"KNOWN"); 
strcpy(TexteSigma[i++],"VARREP"); 
strcpy(TexteSigma[i++],"VARRESID"); 
strcpy(TexteSigma[i++],"IGNORED"); 
strcpy(TexteSigma[i++],"VARINTRA"); 

TexteVari[0]="CST";
TexteVari[1]="SW";
TexteVari[2]="VST";
TexteVari[3]="VB";
TexteVari[4]="VSB";
TexteVari[5]="VSTB";
TexteVari[6]="VTB";
TexteVari[7]="VI";

i=0;
strcpy(TexteCrit[i++],"equal to -2log(likelihood)");  
strcpy(TexteCrit[i++],"equal to the stopping criterion"); 
strcpy(TexteCrit[i++],"equal to the non-weighted sum of squares"); 
strcpy(TexteCrit[i++],"equal to the variance-weighted sum of squares"); 
strcpy(TexteCrit[i++],"equal to the sum of squares weighted by the intra-repetitions-variance"); 
strcpy(TexteCrit[i++],"equal to the non-weighted sum of squares in the context of the variance estimation, divided by n"); 
strcpy(TexteCrit[i++],"equal to sigma**2"); 
strcpy(TexteCrit[i++],""); 
strcpy(TexteCrit[i++],""); 
 strcpy(TexteCrit[i++],""); 
strcpy(TexteCrit[i++],"personal criterion"); 


i=0;
 strcpy(TexteCritCode[i++],"LOGV");
 strcpy(TexteCritCode[i++],"STOPCRIT");
 strcpy(TexteCritCode[i++],"NWSST");
 strcpy(TexteCritCode[i++],"VWSS");
 strcpy(TexteCritCode[i++],"IVWSS");
 strcpy(TexteCritCode[i++],"NWSSB");
 strcpy(TexteCritCode[i++],"SIGMA2");
strcpy(TexteCritCode[i++],""); 
strcpy(TexteCritCode[i++],""); 
 strcpy(TexteCritCode[i++],""); 
 strcpy(TexteCritCode[i++],"MYOWN");

i=0;
 strcpy(TexteFamille[i++], "gaussian");
 strcpy(TexteFamille[i++],"poisson");
 strcpy(TexteFamille[i++],"binomial");
 strcpy(TexteFamille[i++],"bernoulli");
 strcpy(TexteFamille[i++],"multinomial");

 /* Le message qui correspond a CodePuss: */
i=0;
strcpy(TexteMsg[i++],"No errors during execution");  
strcpy(TexteMsg[i++],"The estimation process has not been run: see the warnings ");  
strcpy(TexteMsg[i++],"The regression function could not be calculated at the initial values of the parameters"); 
strcpy(TexteMsg[i++],"The variance function could not be calculated at the initial values of the parameters"); 
strcpy(TexteMsg[i++],"Convergence has not been reached"); 
strcpy(TexteMsg[i++],"The maximum number of iterations has been reached"); 
strcpy(TexteMsg[i++],"An error has occurred when calculating the direction"); 
strcpy(TexteMsg[i++],"One value returned by the variance function is negative or null"); 
strcpy(TexteMsg[i++],"One value returned by the variance function is negative or null at the first iteration"); 
strcpy(TexteMsg[i++],"Invalid trace length"); 
strcpy(TexteMsg[i++],"Invalid input arguments: see the warnings"); 
strcpy(TexteMsg[i++],"A numerical error has occurred"); 
strcpy(TexteMsg[i++],"An error has occurred when calculating the integration system"); 
strcpy(TexteMsg[i++],"The linear system cannot be solved."); 
 strcpy(TexteMsg[i++],"The minimisation criterions are equal at the 3 points of calculation.");


i=0;
CodeMsg[i++] = 0;
CodeMsg[i++] =  200;
CodeMsg[i++] = 156;
CodeMsg[i++] = 157;
CodeMsg[i++] = 21;
CodeMsg[i++] =  22;
CodeMsg[i++] = 23;
CodeMsg[i++] = 145;
CodeMsg[i++] = 158;
CodeMsg[i++] = 112;
CodeMsg[i++] = 120;
CodeMsg[i++] = 150;
CodeMsg[i++] = 307;
CodeMsg[i++] = 73;
 CodeMsg[i++] = 61;



if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recup1C", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */

for (i=0; i< strlen(nom); i++)
  {
  nom[i]=tolower(nom[i]);
  }

if (strcmp(nom, "file") ==0)
  {
    /* le nom du fichier formel */
    return(Modele.NomFichier);
  }

if (strncmp(nom, "message", 4)==0)
  {
    /* Recherche du message correspondant au code */

    k=0;
    while ( (k <nbmess) && ( ResNum[step].CodePuss != CodeMsg[k]))
      k++;

    if (ResNum[step].CodePuss != CodeMsg[k])
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  ResNum[step].CodePuss, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
    else
      return(TexteMsg[k]);
  }

if (strncmp(nom, "stat.crit.type", 11)==0)
  {
    if (CtxNum[step].TypeCritStat == DEFAUT)
      return("default value");

    if (CtxNum[step].TypeCritStat == LEMIEN)
      CtxNum[step].TypeCritStat =11;

        if ((CtxNum[step].TypeCritStat <0) || (CtxNum[step].TypeCritStat >nbcrit))
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  CtxNum[step].TypeCritStat, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteCrit[CtxNum[step].TypeCritStat-1]);
  }


if (strncmp(nom, "stat.crit.code", 11)==0)
  {
    if (CtxNum[step].TypeCritStat == DEFAUT)
      return("default value");

    if (CtxNum[step].TypeCritStat == LEMIEN)
      CtxNum[step].TypeCritStat =11;

        if ((CtxNum[step].TypeCritStat <0) || (CtxNum[step].TypeCritStat >nbcrit))
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  CtxNum[step].TypeCritStat, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteCritCode[CtxNum[step].TypeCritStat-1]);
  }


if (strncmp(nom, "method", 4)==0)
  {
    if (CtxNum[step].Estim == DEFAUT)
      return("default value");

        if ((CtxNum[step].Estim <0) || (CtxNum[step].Estim >nbmeth))
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  CtxNum[step].Estim, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteMeth[CtxNum[step].Estim -1]);
  }


if (strncmp(nom, "n.name", 4)==0)
  {
    if ( Donnees.IndiceN <=0 )	{
      sprintf(message,"No 'count' variable 'N' \n");
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
    return(Donnees.NomX.donnees[Donnees.IndiceN-1]);
  }

if (strncmp(nom, "famil", 4)==0)
  {
    if ( (CtxPuss.Famille == DEFAUT) || (CtxPuss.Famille ==0))
      return("default value");

        if ((CtxPuss.Famille <0) || (CtxPuss.Famille >nbfamille))
	{
	  sprintf(message,
"No message for code %d  of %s \n",
		  CtxPuss.Famille, nom);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteFamille[CtxPuss.Famille -1]);
  }

if (strncmp(nom, "vari.type", 4)==0)
  {
    if (Modele.Vari == DEFAUT)
      return("default value");

        if ((Modele.Vari <0) || (Modele.Vari >nbvari))
	{
	  sprintf(message,
"No message for code %d  of %s \n",
		  Modele.Vari, nom);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteVari[Modele.Vari -1]);
  }


	  

if (strncmp(nom, "w.type", 3)==0)
  {
    if (CtxNum[step].Symm == DEFAUT)
      return("default value");

        if ((CtxNum[step].Symm <0) || (CtxNum[step].Symm >nbsym))
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  CtxNum[step].Symm, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteSym[CtxNum[step].Symm -1]);
  }


if (strncmp(nom, "algo", 3)==0)
  {
    if (CtxPuss.Algo == DEFAUT)
      return("default value");

        if ((CtxPuss.Algo <0) || (CtxPuss.Algo >nbalgo))
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  CtxPuss.Algo, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteAlgo[CtxPuss.Algo -1]);
  }
if (strncmp(nom, "mu.type", 3)==0)
  {
    if (CtxPuss.TypeMu == DEFAUT)
      return("default value");

        if ((CtxPuss.TypeMu <0) || (CtxPuss.TypeMu >nbmu))
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  CtxPuss.TypeMu, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);
	  return(retour);
	}
	  return(TexteMu[CtxPuss.TypeMu -1]);
  }



if (strncmp(nom, "sigma2.type", 3)==0)
  {
    if (CtxPuss.TypeSigma == DEFAUT)
      return("default value");

        if ((CtxPuss.TypeSigma <0) || (CtxPuss.TypeSigma >nbsigma))
	{
	  sprintf(message,
"No message for code %d  of %s at step %d \n",
		  CtxPuss.TypeSigma, nom, step+1);
	  ecrErr(stderr,"recup1C", message, ERRINT);

	  return(retour);
	}
	  return(TexteSigma[CtxPuss.TypeSigma -1]);
  }


sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recup1C", message, ERRARG);
return(retour);

}
/* fin recup1C */


/* ----------------------------------------------------------------------
Recuperation d'un scalaire double
------------------------------------------------------------------------- */
double  recup0D(char *argunom ,int step)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i;
double retour = (double)-1;
/* retourner -1 indique qu'il y a une erreur puisque on n'a jamais
   renvoyer un ngatif: tous les resultats de nls2 sont >0 */

char nom[MAXLGMESS];

strcpy(nom, argunom);

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recup0D", message, ERRARG);
    return(retour);
  }
 step--; 

for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);


if (strncmp(nom, "omega.c1",3)==0)
     return((double)CtxPuss.DirecC);
if (strncmp(nom, "lambda.start",3)==0)
     return((double)CtxPuss.Lambda0);
if (strncmp(nom, "lambda.c1",3)==0)
     return((double)CtxPuss.LambdaC1);
if (strncmp(nom, "lambda.c2",3)==0)
     return((double)CtxPuss.LambdaC2);
if (strncmp(nom, "max.stop.crit", 5)==0)
     return((double)CtxPuss.MaxCritArret);
if (strncmp(nom, "max.err.c2", 10)==0)
     return((double)CtxPuss.MaxDeb);
if (strncmp(nom, "max.err.c1", 10)==0)
     return((double)CtxPuss.MaxErr);
if (strncmp(nom, "max.lambda",5)==0)
     return((double)CtxPuss.MaxLambda);
if (strncmp(nom, "omega.c2",2)==0)
     return((double)CtxPuss.OmegaPas);
if (strncmp(nom, "stat.crit",4)==0)
     return((double)ResNum[step].CritStat);
 if (strncmp(nom, "deviance",4)==0)
   return((double)ResNum[step].Deviance);

if (strncmp(nom, "stop.crit",4)==0)
     return((double)ResNum[step].CritArret);
if (strncmp(nom, "lambda",3)==0)
     return((double)ResNum[step].Lambda);
if (strncmp(nom, "loglik",4)==0)
     return((double)ResNum[step].Log);

if (strncmp(nom, "logfam",4)==0)
     return((double)ResNum[step].LogFamille);
if (strncmp(nom, "norm",3)==0)
     return((double)ResNum[step].Norme);
if (strncmp(nom, "rss.un",4)==0)
     return((double)ResNum[step].ScrNP);
if (strncmp(nom, "rss",3)==0)
     return((double)ResNum[step].Scr);
if (strncmp(nom, "sigma",3)==0)
     return((double)ResNum[step].Sigma);
/*  direc et omega, ne sont  retourns
   que dans le cas sauvegarde par iteration,
   Dans le cas general, ils ne sont pas stocks par nls2:
   ce sont des variables locales a PussIter*/

sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recup0D", message, ERRARG);
return(retour);

}
/* fin recup0D */







/* ----------------------------------------------------------------------
Recuperation d'un scalaire long
------------------------------------------------------------------------- */
long  recup0L(char *argunom ,int step)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i;
long retour = (double)-1;
/* retourner -1 indique qu'il y a une erreur puisque on n'a jamais
   renvoyer un ngatif: tous les resultats de nls2 sont >0 */

char nom[MAXLGMESS];

strcpy(nom, argunom);

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recup0L", message, ERRARG);
    return(retour);
  }
 step--; 

for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);

if (strcmp(nom, "n.obst")==0)
     return((long)Donnees.NbObsT) ;
if (strcmp(nom, "n.obs")==0)
     return((long)Donnees.NbObs) ;
if (strncmp(nom, "n.curves",3)==0)
     return((long)Donnees.NbCourbe);
if (strncmp(nom, "n.x",3)==0)
     return((long)Donnees.NbX);
 if (strncmp(nom, "nb.N",4)==0)
   return((long)Donnees.IndiceN);


if (strncmp(nom, "n.theta", 4)==0)
  return((long)Modele.NomTheta.nbele);
if (strncmp(nom, "n.beta", 4)==0)
  return((long)Modele.NomBeta.nbele);
if (strncmp(nom, "is.odes", 4)==0)
     return((long)Modele.CasSedo);
if (strncmp(nom, "is.v", 4)==0)
     return((long)Modele.YaCalcV);

if (strncmp(nom, "n.steps", 4)==0)
     return((long)CtxPuss.NbEtapes) ;
if (strncmp(nom, "max.iters",5)==0)
     return((long)CtxPuss.MaxIter);
if (strncmp(nom, "code", 4)==0)
     return((long)ResNum[step].CodePuss);
if (strncmp(nom, "n.iters", 4)==0)
     return((long)ResNum[step].NbIter);
if (strncmp(nom, "nh",2)==0)
     return((long)CtxNum[step].NbZ);
if (strncmp(nom, "effic",3)==0)
     return((long)CtxNum[step].Effic);

/* les donnees de GNLCtxInteg */
if ( ( (strncmp(nom, "ci.odes", 4)==0)||
     (strncmp(nom, "x.odes", 4)==0)||
     (strncmp(nom, "n.times", 4)==0)||
     (strncmp(nom, "n.eq.odes", 6)==0)||
       (strncmp(nom, "n.theta.odes", 9)==0)||
     (strncmp(nom, "lg.odes", 6)==0)) &&
     	(GNLCtxInteg.NbThetaSedo ==0)	)		  
      {
	sprintf(message,"%s not valid when the model is not described by an 'odes'!\n",
nom);
	ecrErr(stderr, "recup0L", message, ERRARG);
	return(retour);
      }
if (strncmp(nom, "ci.odes", 4)==0)
  return((long)GNLCtxInteg.IndicCi);
if (strncmp(nom, "x.odes", 4)==0)
  return((long)GNLCtxInteg.IndicX);
if (strncmp(nom, "n.times", 4)==0)
  return((long)GNLCtxInteg.NbJ);
if (strncmp(nom, "n.eq.odes", 6)==0)
  return((long)GNLCtxInteg.NbEq);
if (strncmp(nom, "n.theta.odes", 9)==0)
  return((long)GNLCtxInteg.NbThetaSedo);
if (strncmp(nom, "lg.odes", 6)==0)
  return((long)GNLCtxInteg.LongSys);



sprintf(message,"%s not valid !\n",nom);
ecrErr(stderr, "recup0L", message, ERRARG);
return(retour);

}
/* fin recup0L */





/* ----------------------------------------------------------------------
Recuperation d'un vecteur double
------------------------------------------------------------------------- */
double * recup1D(char *argunom ,int step,int *nbele)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nbele : le nbre d'elements */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i;
double *retour=NULL;

char nom[MAXLGMESS];

strcpy(nom, argunom);

*nbele=0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recup1D", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);

if (strncmp(nom, "weights", 2)==0)
  {
    if ( (Donnees.PoidsT.donnees==NULL) ||
	 (Donnees.PoidsT.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Donnees.PoidsT.nbele;
    return(Donnees.PoidsT.donnees); 
  }

if (strncmp(nom, "theta", 2)==0)
  {
    if ( (Theta[step].Estim.donnees==NULL) ||
	 (Theta[step].Estim.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Theta[step].Estim.nbele;
    return(Theta[step].Estim.donnees); 
  }

if (strncmp(nom, "beta", 2)==0)
  {
    if ( (Beta[step].Estim.donnees==NULL) ||
	 (Beta[step].Estim.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Beta[step].Estim.nbele;
    return(Beta[step].Estim.donnees); 
  }

if (strncmp(nom, "fitted", 3)==0)
  {
    if ( (ResStat[step].Ajustes.Valf.donnees==NULL) ||
	 (ResStat[step].Ajustes.Valf.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].Ajustes.Valf.nbele;
    return(ResStat[step].Ajustes.Valf.donnees); 
  }


if (strncmp(nom, "variance", 3)==0)
  {
    if ( (ResStat[step].Ajustes.VarY.donnees==NULL) ||
	 (ResStat[step].Ajustes.VarY.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].Ajustes.VarY.nbele;
    return(ResStat[step].Ajustes.VarY.donnees); 
  }


if (strncmp(nom, "mu3", 3)==0)
  {
    if ( (ResStat[step].Mu3.donnees==NULL) ||
	 (ResStat[step].Mu3.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].Mu3.nbele;
    return(ResStat[step].Mu3.donnees); 
  }
if (strncmp(nom, "mu4", 3)==0)
  {
    if ( (ResStat[step].Mu4.donnees==NULL) ||
	 (ResStat[step].Mu4.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].Mu4.nbele;
    return(ResStat[step].Mu4.donnees); 
  }



if (strncmp(nom, "eta", 3)==0)
  {
    if ( (ResStat[step].EquN.ValEta.donnees==NULL) ||
	 (ResStat[step].EquN.ValEta.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].EquN.ValEta.nbele;
    return(ResStat[step].EquN.ValEta.donnees); 
  }


if (strncmp(nom, "z", 1)==0)
  {
    if ( (ResStat[step].ValZ.donnees==NULL) ||
	 (ResStat[step].ValZ.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].ValZ.nbele;
    return(ResStat[step].ValZ.donnees); 
  }




if (strncmp(nom, "residuals", 4)==0)
  {
    if ( (ResStat[step].Residus.donnees==NULL) ||
	 (ResStat[step].Residus.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].Residus.nbele;
    return(ResStat[step].Residus.donnees); 
  }

if (strncmp(nom, "s.residuals", 4)==0)
  {
    if ( (ResStat[step].ResidusR.donnees==NULL) ||
	 (ResStat[step].ResidusR.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResStat[step].ResidusR.nbele;
    return(ResStat[step].ResidusR.donnees); 
  }


if (strncmp(nom, "dev.residuals", 4)==0)
  {
    if ( (ResNum[step].ResidusDev.donnees==NULL) ||
	 (ResNum[step].ResidusDev.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=ResNum[step].ResidusDev.nbele;
    return(ResNum[step].ResidusDev.donnees); 
  }

if (strncmp(nom, "y1", 2)==0)
  {
    if ( (Donnees.StatDon.Y1.donnees==NULL) ||
	 (Donnees.StatDon.Y1.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Donnees.StatDon.Y1.nbele ;
    return(Donnees.StatDon.Y1.donnees);
  }


if (strncmp(nom, "y2", 2)==0)
  {
    if ( (Donnees.StatDon.Y2.donnees==NULL) ||
	 (Donnees.StatDon.Y2.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Donnees.StatDon.Y2.nbele ;
    return(Donnees.StatDon.Y2.donnees);
  }
if (strncmp(nom, "s2", 2)==0)
  {
    if ( (Donnees.StatDon.S2.donnees==NULL) ||
	 (Donnees.StatDon.S2.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Donnees.StatDon.S2.nbele ;
    return(Donnees.StatDon.S2.donnees);
  }

if (strncmp(nom, "y", 1)==0)
  {
    if ( (Donnees.ValY.donnees==NULL) ||
	 (Donnees.ValY.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Donnees.ValY.nbele ;
    return(Donnees.ValY.donnees);
  }

if (strncmp(nom, "replications", 3)==0)
  {
    if ( (Donnees.NbRepet.donnees==NULL) ||
	 (Donnees.NbRepet.nbele <= 0))
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup1D", message, ERRFOUND);
	return(retour);
      }
    *nbele=Donnees.NbRepet.nbele ;
    /* Convertir en double */
    CREER_T1(retour, *nbele, TDouble);
    for (i=0; i< *nbele; i++)
      retour[i]= (double)Donnees.NbRepet.donnees[i];
    return(retour);

  }
sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recup1D", message, ERRARG);
return(retour);

}
/* fin recup1D */

    

/* ----------------------------------------------------------------------
Recuperation d'un tableau 3 dim
En l'occurrence, allouer et recuperer les FSedo 
------------------------------------------------------------------------- */
double ***recup3D(char *argunom ,int step,int *nb1, int *nb2, int *nb3)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nb1, nb2, nb3 : le nbre d'elements */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i;
double ***retour= (double ***)NULL;

char nom[MAXLGMESS];

strcpy(nom, argunom);

*nb1= *nb2 = *nb3 =0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recup3D", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);


if (strncmp(nom, "fodes", 4)==0)
  {
    if (ResStat[step].Sedo.FSedo==NULL)
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup3D", message, ERRFOUND);
	return(retour);
      }
    *nb1=(int)Donnees.NbObs;
    *nb2=GNLCtxInteg.NbJ;
    *nb3=GNLCtxInteg.NbEq;
    return(test3D(nom,ResStat[step].Sedo.FSedo, *nb1, *nb2, *nb3));
  }

if (strncmp(nom, "d.fodes", 4)==0)
  {
    if (ResStat[step].Sedo.DFSedo==NULL)
      {
	sprintf(message,"No %s found \n",nom);
	ecrErr(stderr, "recup3D", message, ERRFOUND);
	return(retour);
      }
    *nb1=(int)Donnees.NbObs;
    *nb2=GNLCtxInteg.NbJ;
    *nb3=GNLCtxInteg.LongSys - GNLCtxInteg.NbEq;
    return(test3D(nom,ResStat[step].Sedo.DFSedo, *nb1, *nb2, *nb3));
  }

sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recup3D", message, ERRARG);
return(retour);

}
/* fin recup3D */








