/* +++++++++++++++++++++++++++++++++++++++++++++++++++
les define et les includes 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <ctype.h>
#include "nls2C.h"
/* pour crire les messages d'erreur */
extern void ecrErr(FILE *fic, char * nomcall , char *message, int Code);


/* ----------------------------------------------------------------------
Recuperation des valeurs stockes dans les structures
(toutes sont dcl en global) de nls2
------------------------------------------------------------------------- */

/* ----------------------------------------------------------------------
Recuperation d'un vecteur double d'un resultat par iter, i.e renvoie
une matrice, dont la 1iere dimension est le nombre d'iterations
sauvegardes, et la 2ieme la taille du vecteur
------------------------------------------------------------------------- */
double ** recupIt2D(char *argunom ,int step, int *nbitsv, int *nbele)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nbitsv : le nombre d'iterations sauvegardes */
/* nbele : le nbre d'elements */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i, iter;
double **retour=(double **)NULL;
char nom[MAXLGMESS];

strcpy(nom, argunom);

*nbitsv=0;
*nbele=0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recupIt2D", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */

if (NbItSv[step] ==0)
  {
    sprintf(message,"No iteration results saved at step %d!\n",   step+1);
    ecrErr(stderr,"recupIt2D", message, ERRFOUND);
     return(retour);
  }
*nbitsv = NbItSv[step];

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);

if (strncmp(nom, "theta", 2)==0)
  {
    if (GNLControle.VouluIt.Estim != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved \n",	nom);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	return(NULL);
      }

    *nbele=ItTheta[step][0].nbele ;
    /* Allouer la matrice resultat : la 2 ieme colonne contiendra
       des pointeurs */
    CREER_T2(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItTheta[step][iter].donnees==NULL) ||
	    (ItTheta[step][iter].nbele <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItTheta[step][iter].donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin if (strncmp(nom, "theta", 2)==0) */

if (strncmp(nom, "beta", 2)==0)
  {
    if (GNLControle.VouluIt.Estim != VRAI)
      {
	sprintf(message,
"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	return(NULL);
      }

    *nbele=ItBeta[step][0].nbele ;
    /* Allouer la matrice resultat : la 2 ieme colonne contiendra
       des pointeurs */
    CREER_T2(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItBeta[step][iter].donnees==NULL) ||
	    (ItBeta[step][iter].nbele <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItBeta[step][iter].donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin beta */

if (strncmp(nom, "response", 4)==0)
  {
    if (GNLControle.VouluIt.Ajustes != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved \n",	nom);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	return(NULL);
      }

    *nbele=ItStat[step][0].Ajustes.Valf.nbele ;
    /* Allouer la matrice resultat : la 2 ieme colonne contiendra
       des pointeurs */
    CREER_T2(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].Ajustes.Valf.donnees==NULL) ||
	    (ItStat[step][iter].Ajustes.Valf.nbele <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].Ajustes.Valf.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin response */


if (strncmp(nom, "variance", 3)==0)
  {
    if (GNLControle.VouluIt.Ajustes != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	return(NULL);
      }

    *nbele=ItStat[step][0].Ajustes.VarY.nbele ;
    /* Allouer la matrice resultat : la 2 ieme colonne contiendra
       des pointeurs */
    CREER_T2(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].Ajustes.VarY.donnees==NULL) ||
	    (ItStat[step][iter].Ajustes.VarY.nbele <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].Ajustes.VarY.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin variance */


if (strncmp(nom, "eta", 3)==0)
  {
    if (GNLControle.VouluIt.EquN != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	return(NULL);
      }

    *nbele=ItStat[step][0].EquN.ValEta.nbele ;
    /* Allouer la matrice resultat : la 2 ieme colonne contiendra
       des pointeurs */
    CREER_T2(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].EquN.ValEta.donnees==NULL) ||
	    (ItStat[step][iter].EquN.ValEta.nbele <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].EquN.ValEta.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin eta */

if (strncmp(nom, "direc", 3)==0)
  {
    if (GNLControle.VouluIt.ResNum != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
	return(NULL);
      }
    *nbele = ItDirec[step][0].nbele ;
    /* Allouer la matrice resultat : la 2 ieme colonne contiendra
       des pointeurs */
    CREER_T2(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	    /* A l'iter 0, il n'a pas ete calcul : 
       il contiendra mysignaling_nan(nn)=DBL_MAX ici */
	if ((ItDirec[step][iter].donnees==NULL) ||
		    (ItDirec[step][iter].nbele <= 0))
		  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt2D", message, ERRFOUND);
		    return(NULL);
		  }
		retour[iter] = ItDirec[step][iter].donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin direc */



sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recupIt2D", message, ERRARG);

return(retour);

}
/* fin recupIt2D */






/* ----------------------------------------------------------------------
Recuperation d'une matrice double d'un resultat par iter, i.e renvoie
un tableau a 3 dim, dont la 1iere dimension est le nombre d'iterations
sauvegardes, et les autres, celles de la matrice.
------------------------------------------------------------------------- */
double *** recupIt3D(char *argunom ,int step, int *nbitsv, int *nblig, int *nbcol)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nbitsv : le nombre d'iterations sauvegardes */
     /* nblig, nbcol : les dimensions de la matrice */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i, iter;
double ***retour=(double ***)NULL;
char nom[MAXLGMESS];

strcpy(nom, argunom);

*nbitsv= *nblig = *nbcol = 0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recupIt3D", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */

if (NbItSv[step] ==0)
  {
    sprintf(message,"No iteration results saved at step %d!\n",   step+1);
    ecrErr(stderr,"recupIt3D", message, ERRFOUND);
     return(retour);
  }
*nbitsv = NbItSv[step];

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);

/* Test a mettre avant if (strncmp(nom, "d",1) */
if (strncmp(nom, "d.resp", 3)==0)
  {
    if (GNLControle.VouluIt.FctSensib != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	return(NULL);
      }

    *nblig=ItStat[step][0].FctSensib.DValf.nblig ;
    *nbcol=ItStat[step][0].FctSensib.DValf.nbcol ;

    /* Allouer le tableau resultat : la 2 ieme dim contiendra
       des pointeurs */
    CREER_T3(retour, *nbitsv ,  TDouble);

    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].FctSensib.DValf.donnees==NULL) ||
	    (ItStat[step][iter].FctSensib.DValf.nblig <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].FctSensib.DValf.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin if (strncmp(nom, "d.resp" .. */

/* Test a mettre avant if (strncmp(nom, "d",1) */
if (strncmp(nom, "d.theta.vari", 3)==0)
  {
    if (GNLControle.VouluIt.FctSensib != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	return(NULL);
      }

    *nblig=ItStat[step][0].FctSensib.DVarYTheta.nblig ;
    *nbcol=ItStat[step][0].FctSensib.DVarYTheta.nbcol ;

    /* Allouer le tableau resultat : la 2 ieme dim contiendra
       des pointeurs */
    CREER_T3(retour, *nbitsv ,  TDouble);

    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].FctSensib.DVarYTheta.donnees==NULL) ||
	    (ItStat[step][iter].FctSensib.DVarYTheta.nblig <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].FctSensib.DVarYTheta.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin if (strncmp(nom, "d.theta.vari" .. */


/* Test a mettre avant if (strncmp(nom, "d",1) */
if (strncmp(nom, "d.beta.vari", 3)==0)
  {
    if (GNLControle.VouluIt.FctSensib != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	return(NULL);
      }

    *nblig=ItStat[step][0].FctSensib.DVarYBeta.nblig ;
    *nbcol=ItStat[step][0].FctSensib.DVarYBeta.nbcol ;

    /* Allouer le tableau resultat : la 2 ieme dim contiendra
       des pointeurs */
    CREER_T3(retour, *nbitsv ,  TDouble);

    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].FctSensib.DVarYBeta.donnees==NULL) ||
	    (ItStat[step][iter].FctSensib.DVarYBeta.nblig <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].FctSensib.DVarYBeta.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin if (strncmp(nom, "d.beta.vari" .. */




if (strncmp(nom, "b", 1)==0)
  {
    if (GNLControle.VouluIt.EquN != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	return(NULL);
      }

    *nblig=ItStat[step][0].EquN.ValB.nblig ;
    *nbcol=ItStat[step][0].EquN.ValB.nbcol ;

    /* Allouer le tableau resultat : la 2 ieme dim contiendra
       des pointeurs */
    CREER_T3(retour, *nbitsv ,  TDouble);

    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].EquN.ValB.donnees==NULL) ||
	    (ItStat[step][iter].EquN.ValB.nblig <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].EquN.ValB.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin if (strncmp(nom, "B", 1)==0) */


if (strncmp(nom, "d", 1)==0)
  {
    if (GNLControle.VouluIt.EquN != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	return(NULL);
      }

    *nblig=ItStat[step][0].EquN.ValD.nblig ;
    *nbcol=ItStat[step][0].EquN.ValD.nbcol ;

    /* Allouer le tableau resultat : la 2 ieme dim contiendra
       des pointeurs */
    CREER_T3(retour, *nbitsv ,  TDouble);

    for (iter =0; iter<*nbitsv; iter++)
      {
	if ((ItStat[step][iter].EquN.ValD.donnees==NULL) ||
	    (ItStat[step][iter].EquN.ValD.nblig <= 0))
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt3D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].EquN.ValD.donnees;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin if (strncmp(nom, "D", 1)==0) */

sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recupIt3D", message, ERRARG);
return(retour);

}
/* fin recupIt3D */



/* ----------------------------------------------------------------------
Recuperation d'un FSedo, tableau a 3 dim double d'un resultat par iter, i.e renvoie
un tableau a 4 dim, dont la 1iere dimension est le nombre d'iterations
sauvegardes, et les autres, des pointeurs sur le tableau a 3 dim.
------------------------------------------------------------------------- */
double **** recupIt4D(char *argunom ,int step, int *nbitsv, 
     int *nb1, int *nb2, int *nb3)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nbitsv : le nombre d'iterations sauvegardes */
     /* nb1, nb2, nb3: les dimensions du tableau */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i, iter;
double ****retour=(double ****)NULL;
char nom[MAXLGMESS];

strcpy(nom, argunom);

*nbitsv= *nb1 = *nb2 = *nb3 = 0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recupIt4D", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */

if (NbItSv[step] ==0)
  {
    sprintf(message,"No iteration results saved at step %d!\n",   step+1);
    ecrErr(stderr,"recupIt4D", message, ERRFOUND);
     return(retour);
  }
*nbitsv = NbItSv[step];

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);


if (strncmp(nom, "fodes", 3)==0)
  {
    if (GNLControle.VouluIt.Sedo != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt4D", message, ERRFOUND);
	return(NULL);
      }

    *nb1=(int)Donnees.NbObs;
    *nb2=GNLCtxInteg.NbJ;
    *nb3=GNLCtxInteg.NbEq;


    /* Allouer le tableau resultat : la 2 ieme dim contiendra
       des pointeurs */
    CREER_T4(retour, *nbitsv ,  TDouble);

    for (iter =0; iter<*nbitsv; iter++)
      {
	if (ItStat[step][iter].Sedo.FSedo==NULL) 
	  {
	   sprintf(message,
"The structure '%s' does not exist or has not been calculated at iteration %d, step %d, \n",
		    nom, iter, step+1);
	ecrErr(stderr,"recupIt4D", message, ERRFOUND);
	    return(NULL);
	  }
	retour[iter] = ItStat[step][iter].Sedo.FSedo;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin if (strncmp(nom, "fodes" .. */

sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recupIt4D", message, ERRARG);

return(retour);

}
/* fin recupIt4D */


/* ----------------------------------------------------------------------
Recuperation d'un resultat par iteration:
scalaire qui devient un vecteur double, la 1iere dimension
tant le nbre d'iterations sauvegardes
------------------------------------------------------------------------- */
double * recupIt1D(char *nom ,int step,int *nbitsv)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nbitsv : le nbre d'elements, i.e le nbre d'iterations sauvegardes */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i, iter;
double *retour=(double *)NULL;

*nbitsv=0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recupIt1D", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */


if (NbItSv[step] ==0)
  {
    sprintf(message,"No iteration results saved at step %d!\n",   step+1);
    ecrErr(stderr,"recupIt1D", message, ERRFOUND);
     return(retour);
  }
*nbitsv = NbItSv[step];

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);

if (strncmp(nom, "lambda", 3)==0)
  {
    if ( GNLControle.VouluIt.ResNum != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt1D", message, ERRFOUND);
	return(NULL);
      }
    /* Allouer le vecteur resultat */

    CREER_T1(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	retour[iter] = ItNum[step][iter].Lambda;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin lambda */

if (strncmp(nom, "omega", 3)==0)
  {
    if ( GNLControle.VouluIt.ResNum != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt1D", message, ERRFOUND);
	return(NULL);
      }
    /* A l'iter 0, il n'a pas ete calcul : 
       il contiendra mysignaling_nan(nn)=DBL_MAX ici */
    return(ItOmega[step]);
  } /* fin omega */

if (strncmp(nom, "stop.crit", 3)==0)
  {
    if ( GNLControle.VouluIt.ResNum != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt1D", message, ERRFOUND);
	return(NULL);
      }
    /* Allouer le vecteur resultat */
    CREER_T1(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	retour[iter] = (double)ItNum[step][iter].CritArret;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin stop.crit */


if (strncmp(nom, "stat.crit", 3)==0)
  {
    if ( GNLControle.VouluIt.ResNum != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt1D", message, ERRFOUND);
	return(NULL);
      }
    /* Allouer le vecteur resultat */
    CREER_T1(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	retour[iter] = (double)ItNum[step][iter].CritStat;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin stat.crit */




if (strncmp(nom, "sigma2", 3)==0)
  {
    if ( GNLControle.VouluIt.ResNum != VRAI)
      {
	sprintf(message,"You have not asked %s to be saved. \n",	nom);
	ecrErr(stderr,"recupIt1D", message, ERRFOUND);
	return(NULL);
      }
    /* Allouer le vecteur resultat */
    CREER_T1(retour, *nbitsv ,  TDouble);
    for (iter =0; iter<*nbitsv; iter++)
      {
	retour[iter] = (double)ItNum[step][iter].Sigma;

      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin sigma2 */



sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recupIt1D", message, ERRARG);
return(retour);

}
/* fin recupIt1D */







/* ----------------------------------------------------------------------
Recuperation d'un resultat par iteration:
scalaire qui devient un vecteur long, la 1iere dimension
tant le nbre d'iterations sauvegardes
------------------------------------------------------------------------- */
long int * recupIt1L(char *nom ,int step,int *nbitsv)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom en anglais de ce qu'on veut en sortie */
/* step: le no de step a partir de 1 */
/* ............... Arguments de sortie   ....................... */
/* nbitsv : le nbre d'elements, i.e le nbre d'iterations sauvegardes */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int i, iter;
long int *retour=(long int *)NULL;

*nbitsv=0;

if ((step < 1) || (step > CtxPuss.NbEtapes))
  {
    sprintf(message,
"Step is %d. It should be between 1 and %d !\n",
	    step, CtxPuss.NbEtapes);
    ecrErr(stderr,"recupIt1L", message, ERRARG);
    return(retour);
  }
 step--; /* mise a partir de zero */


if (NbItSv[step] ==0)
  {
    sprintf(message,"No iteration results saved at step %d!\n",   step+1);
    ecrErr(stderr,"recupIt1L", message, ERRFOUND);
     return(retour);
  }
*nbitsv = NbItSv[step];

 /* On convertit nom en min pour pouvoir tester en une seule fois
    les min et maj */
for (i=0; i< strlen(nom); i++)
  nom[i]=tolower(nom[i]);

if (strncmp(nom, "iter", 3)==0)
  {
    /* Les nos d'iterations sont tjrs sauvegardes, mme si 
       ItNum[step][0].NbIter == NULL */
    /* Allouer le vecteur resultat */

    CREER_T1(retour, *nbitsv ,  TLongInt);
    for (iter =0; iter<*nbitsv; iter++)
      {
	retour[iter] = ItNum[step][iter].NbIter;
      } /* fin for (iter =0; iter<*nbitsv; iter++) */  
    return(retour);
  } /* fin iter */


sprintf(message,"%s not valid  !\n",nom);
ecrErr(stderr, "recupIt1L", message, ERRARG);

return(retour);

}
/* fin recupIt1L */

    
