
/* ------------------------------------------------------
ab:  avril 2001
Programme appel par le main de l'utilisateur

Correspond aux fonctions SPlus renls2 et rexnls2:
L'utilisateur peut entrer de nouvelles valeurs pour :
- les valeurs init. des theta/beta
- les valeurs des pbis
- les variables indpendantes (les x)
- les valeurs observes de la rponse
- les poids sur les observations
Ce programme doit tre appel aprs un appel  nls2C.
Il est destin  tre appel dans une boucle o une ou pls
des valeurs ci-dessus change.
Les dimensions doivent tre les mmes que lors de l'appel   nls2C.

Ici, par dfaut, on ne change pas les valeurs init. du prcdent appel
 renls2C ou nls2C. Si l'utilisateur veut comme
 val. init. des paramtres les prcdentes
valeurs estimes, ou, pour la rponse, les prcdentes
valeurs ajustes, ce sera  lui d'aller les rcuprer et de les
passer en argu.
A lui aussi, de mettre les fitted (val. ajustes de la rponse)
en dim n;
Les val. init. des paramtres doivent tre en dim pmult,qmult
(NbCourbe*  nombre de parametres du modele)
et  la rponse en dim n (nbre total d'observations).
 ----------------------------------------------------------- */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++
les define et les includes 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include "nls2C.h"

/* Fonction externes */
extern TShortInt CreerVect( TShortInt nbele, TVect *pvect);

/* ----------------------------------------------------------------
renls2
------------------------------------------------------------------*/

int renls2C(double *ThetaInit, double *BetaInit, 
	    double *GamF, double *GamV,
            double **x,
	    double *y,
	    double *poids,
	    int ip)
{
  /* -----  Fonctions externes appeles ------------------- */

/* pour crire les messages d'erreur */
extern void ecrErr(FILE *fic, char * nomcall , char *message, int Code);

/* Fonctions de nls2 */
extern TShortInt NLVDon(TDonnees *Donnees, TShortInt *Code);
extern TShortInt NLVDonM(TShortInt NbTheta,
			 TShortInt NbBeta,
			 TVectLong  *NbObsC,
			 TShortInt *Code);
extern TShortInt NLAutres(TDonnees *Donnees,
			  TCtxPuss *CtxPuss,
			  TCtxNum CtxNum[],
			  TParam Theta[] , TParam Beta[],
			  TContr CTheta[], TContr CBeta[],
			  TContr CThetaE[], TContr CBetaE[],
			  TResNum ResNum[],
			  TResStat ResStat[]);

  /* ------------------ declarations des structures ----------------*/

TLongInt  NbTheta=0, NbBeta=0, NbGamF=0, NbGamV=0;
TShortInt  NbX, mycode ;
TLongInt   NbObsT ; 
TShortInt ietap ; /* indice de la dernire tape calcule */
 /* logiques qui indiquent ce qu'on change par rapport au prcdent appel */
TLogic chtheta=FAUX,
  chbeta=FAUX,
  chgamf=FAUX,
  chgamv=FAUX,
  chx=FAUX, chy=FAUX, chpoids=FAUX;
int Code=OK;

/* Pour les messages d'erreur */
char message[MAXLGMESS];

/* Fonctions appeles */
TShortInt rego2C(int ip);

/* ------------------ debut du corps ------------------------------ */


ietap =  CtxPuss.NbEtapes-1;
if ((ietap <0) || (ietap > 2))
  {
    sprintf(message,
"Invalid value for the step index: %d:  have you called 'nls2C' with success before on the same model? \n", ietap);
    ecrErr(stderr,"renls2C",message,  ERRFOUND);
	return((int)ERRFOUND);
  }


/* ------------ Recupration des paramtres ------------------------*/

/*---  Les Theta   --- */
NbTheta = Theta[ietap].Estim.nbele ;
if ( NbTheta <= 0)
  {
    sprintf(message,
"Invalid value for the number of parameters 'theta': %ld:  have you called 'nls2C' with success before on the same model? \n", NbTheta);
    ecrErr(stderr,"renls2C",message,  ERRFOUND);
	return((int)ERRFOUND);
  }

if (ThetaInit != (double *)NULL)
  {
   chtheta=VRAI;
    CtxPuss.Theta0.donnees = ThetaInit;
  }


/*---  Les Beta   --- */
NbBeta = Beta[ietap].Estim.nbele ;

if (BetaInit != (double *)NULL)
  {
    /* des beta sont en argu  */
    if   (NbBeta <= 0)
      {
	/*  des beta sont en argu alors qu'il n'y en a pas dans le modele */
	ecrErr(stderr,"renls2C",
"No estimated values for 'beta':  have you called 'nls2C' with success before on the same model? \n", ERRFOUND);
	return((int)ERRFOUND);
      } /* fin if   (NbBeta <= 0) */
    else
      {
	CtxPuss.Beta0.donnees = BetaInit;
	chbeta=VRAI;
      }
  } /* fin if (BetaInit != (double *)NULL) */


/*---  Les GamF   --- */
NbGamF = Modele.GamF.nbele ;

if (GamF != (double *)NULL)
  {
    /* des gamf sont en argu  */
    if   (NbGamF <= 0)
      {
	/*  des gamf sont en argu alors qu'il n'y en a pas dans le modele */
	ecrErr(stderr,"renls2C",
"No preceding values found for 'gamf':  have you called 'nls2C' with success before on the same model? \n", ERRFOUND);
	return(ERRFOUND);
      } /* fin if   (NbGamF <= 0) */
    else
      {
	Modele.GamF.donnees = GamF;
	chgamf=VRAI;
      }
  } /* fin if (GamF != (double *)NULL) */


/*---  Les GamV   --- */
NbGamV = Modele.GamV.nbele ;

if (GamV != (double *)NULL)
  {
    /* des gamv sont en argu  */
    if   (NbGamV <= 0)
      {
	/*  des gamv sont en argu alors qu'il n'y en a pas dans le modele */
	ecrErr(stderr,"renls2C",
"No preceding values found for 'gamv':  have you called 'nls2C' with success before on the same model? \n", ERRFOUND);
	return(ERRFOUND);
      } /* fin if   (NbGamV <= 0) */
    else
      {
	Modele.GamV.donnees = GamV ;
	chgamv=VRAI;
      }
  } /* fin if (GamV != (double *)NULL) */
      
/* ----------- les x --------------------------------- */

NbX = Donnees.NbX;
NbObsT= Donnees.NbObsT;
if (x != (double **)NULL)
  {
   chx=VRAI;
    /* Des x sont en argu */
    Donnees.XObsT.donnees=x;
  }

/* ----------- les Y --------------------------------- */

if (y != (double *)NULL)
  {
   chy=VRAI;
    /* Des y sont en argu */
      Donnees.ValY.donnees =y;
  }

/* ----------- les poids --------------------------------- */

if (poids != (double *)NULL)
  {
   chpoids=VRAI;
    /* Des poids sont en argu */
      Donnees.PoidsT.donnees =poids;
  }



/* --------- reinitialisation des structures de NL --------------*/
/* reinitialisation des compteurs de warnings */
 GNLControle.CWarAna=0 ;  /* nbre de warnings pour l'analyseur-derivateur */
 GNLControle.CWarInt=0 ;  /* nbre de warnings pour l'integrateur */
 GNLControle.CWarMet=0 ;  /* nbre de warnings de type methodologique */
 GNLControle.CWarNum=0 ;  /* nbre de warnings de type numerique */
 GNLControle.CWarTot=0 ;  /* nbre de warnings tous types */

if ((chx==VRAI) || (chy==VRAI) || (chpoids==VRAI))
  {
  /* init, verif des donnees */
    Code = (int)NLVDon( &(Donnees), &mycode);
    if ((Code != (int)OK) || (mycode !=(int)OK))
      {
	sprintf(message,
"\nError  %d in the data reinitialisation, program 'NLVDon' \n",
		Code);
	ecrErr(stderr,"renls2C", message, Code);
	return(Code);
      } /* fin if ((Code != (int)OK) || (mycode !=(int)OK)) */
    /* Verif de la concordance entre donnees et modele: le nombre d'observations
   doit rester sup au nombre de parametres */
    Code = (int)NLVDonM(
		   Modele.NomTheta.nbele, Modele.NomBeta.nbele, 
		   &(Donnees.NbObsC), &mycode);
    if ((Code != (int)OK) || (mycode !=(int)OK))
      {
	sprintf(message,
"\nError  %d in the data reinitialisation, program 'NLVDonM' \n",
		Code);
	ecrErr(stderr,"renls2C", message, Code);
	return(Code);
      } /* fin if ((Code != (int)OK) || (mycode !=(int)OK)) */

  } /* fin if ((chx==VRAI) || (chy==VRAI) || (chpoids==VRAI)) */


/* --------- On lance NL ---------------------- */
if ( (Code=rego2C(ip)) != (int)OK)
  return(Code);

/* --------- On fait les  calculs supplementaires---------- */
Code= (int)NLAutres(&Donnees, &CtxPuss, CtxNum, 
                  Theta, Beta, CTheta, CBeta, CThetaE, CBetaE,
                  ResNum, ResStat);

if (Code !=(int)OK)
  {
    ecrErr(stderr,"renls2C", 
"\nError when the wanted structures, program 'NLAutres' \n",
		Code);
	return(Code);
  } /* fin if ((Code != (int)OK) */

return((int)OK);

}

/* -------------- Fin fonction renls2C --------------------- */



/* ------------------------------------------------------------------
rego2C: programme qui lance l'excution de NL
------------------------------------------------------------------- */
TShortInt rego2C(int ip)
{
  /* -----  Fonctions externes appeles ------------------- */

/* pour crire les messages d'erreur */
extern void ecrErr(FILE *fic, char * nomcall , char *message, int Code);

extern TShortInt NLEtape(TShortInt Etape, 
			 TDonnees *Donnees, 
			 TModele *Modele, TCtxPuss *CtxPuss, TCtxNum *CtxNum,
			 TVect *ThetaPred1, TVect *ThetaPred2, 
			 TVect *BetaPred1, TVect *BetaPred2,
			 TVect *Valf1, TMat *DValf1,
			 TParam  *Theta, TParam  *Beta, 
			 TContr *CTheta, TContr *CBeta, 
			 TContr *CThetaE, TContr *CBetaE, 
			 TResNum *ResNum, TResStat *ResStat,
			 TShortInt *NbItSv, TResNum *ItNum, 
			 TResStat *ItStat, 
			 TVect *ItTheta, TVect *ItBeta, TVect *ItDirec,
			 TDouble *ItOmega);

  /* ------------------ declarations des structures ----------------*/
TShortInt NbCodesErr=12;
TShortInt CodesErr[12];
TShortInt   i, Etape, IndEtap;
TVect ThetaPred1, ThetaPred2, BetaPred1, BetaPred2;
FILE *SortImp_sv, *SortWE_sv;
 TLogic fatal=FAUX; /* indique si l'erreur de NL est fatale */
int Code=OK;


/* ------------------ debut du corps ------------------------------ */
/* Initialisations */

/* CodesErr: les codes de retour de NL pour lesquels les sorties sont non valides */
/* erreur fatale dans ces cas */
CodesErr[0]=NONFAIT;
CodesErr[1]=ERRTYPES;
CodesErr[2]=ERRTRACE;
CodesErr[3]=ERRVINIT;
CodesErr[4]=ERWT;
CodesErr[5]=ERWM;
CodesErr[6]=ERWI;
CodesErr[7]=ERWN;
CodesErr[8]=ERRCMU;
CodesErr[9]=EPBALLOC;
CodesErr[10]=ERRALLOC;
CodesErr[11] = ERRSYS;

/* On ote les ip car elles s'criraient  cahque boucle */
if (ip == 0)
  {
    SortImp_sv = GNLControle.SortImp;
    SortWE_sv = GNLControle.SortWE;
    GNLControle.SortImp=NULL;
    GNLControle.SortWE=NULL;
  }

/* on cree des vecteurs bidon pour pouvoir faire l'appel a NL
avec les memes arguments, qu'il y ait des valeurs precedemment estimees
ou non */
CreerVect((TShortInt)0, &ThetaPred1);
CreerVect((TShortInt)0, &ThetaPred2);
CreerVect((TShortInt)0, &BetaPred1);
CreerVect((TShortInt)0, &BetaPred2);


/* APPEL  DE NLEtape */
  for(Etape=1; Etape<=CtxPuss.NbEtapes; Etape++)
    {
    IndEtap = Etape - 1; /* indice dans les structures */

    switch(Etape)
      {
      case 1:
        Code=(int)NLEtape(Etape, &Donnees, &Modele, &CtxPuss, &(CtxNum[IndEtap]),
              &ThetaPred1, &ThetaPred2, &BetaPred1, &BetaPred2,
              &(ResStat[IndEtap].Ajustes.Valf),
              &(ResStat[IndEtap].FctSensib.DValf),
              &(Theta[IndEtap]), &(Beta[IndEtap]), 
              &(CTheta[IndEtap]), &(CBeta[IndEtap]),
              &(CThetaE[IndEtap]), &(CBetaE[IndEtap]),
              &(ResNum[IndEtap]), &(ResStat[IndEtap]),
              &(NbItSv[IndEtap]),
              ItNum[IndEtap], ItStat[IndEtap],
              ItTheta[IndEtap], ItBeta[IndEtap],
              ItDirec[IndEtap], ItOmega[IndEtap]);


         break;
      case 2:
         Code=(int)NLEtape(Etape, &Donnees, &Modele, &CtxPuss, &(CtxNum[IndEtap]),
              &(Theta[IndEtap-1].Estim), &ThetaPred2, 
              &(Beta[IndEtap-1].Estim), &BetaPred2,
              &(ResStat[IndEtap].Ajustes.Valf),
              &(ResStat[IndEtap].FctSensib.DValf),
              &(Theta[IndEtap]), &(Beta[IndEtap]), 
              &(CTheta[IndEtap]), &(CBeta[IndEtap]),
              &(CThetaE[IndEtap]), &(CBetaE[IndEtap]),
              &(ResNum[IndEtap]), &(ResStat[IndEtap]),
              &(NbItSv[IndEtap]),
              ItNum[IndEtap], ItStat[IndEtap],
              ItTheta[IndEtap], ItBeta[IndEtap],
              ItDirec[IndEtap], ItOmega[IndEtap]);

         break;
      default:
        Code=(int)NLEtape(Etape, &Donnees, &Modele, &CtxPuss, &(CtxNum[IndEtap]),
               &(Theta[IndEtap-1].Estim), &(Theta[IndEtap-2].Estim),
               &(Beta[IndEtap-1].Estim), &(Beta[IndEtap-2].Estim),
              &(ResStat[IndEtap-1].Ajustes.Valf),
              &(ResStat[IndEtap-1].FctSensib.DValf),
              &(Theta[IndEtap]), &(Beta[IndEtap]), 
              &(CTheta[IndEtap]), &(CBeta[IndEtap]),
              &(CThetaE[IndEtap]), &(CBetaE[IndEtap]),
              &(ResNum[IndEtap]), &(ResStat[IndEtap]),
              &(NbItSv[IndEtap]),
              ItNum[IndEtap], ItStat[IndEtap],
              ItTheta[IndEtap], ItBeta[IndEtap],
              ItDirec[IndEtap], ItOmega[IndEtap]);

        break;

      } /* fin du switch */
    if (Code !=(int)OK)
      {
      /* Impression en cas d'erreur lors de l'estimation */
      /* ----------------------------------------------------------- */
	ecrErr(stderr,"renls2C","", Code);
      for (i=0; i<NbCodesErr; i++)
        {
        if (Code == (int)CodesErr[i])
          {
	    fatal = VRAI;
	    break ; /* sortie de la boucle sur i=0; i<NbCodesErr; fatale erreur */
	  }
        } /* fin i=0; i<NbCodesErr; i++ */
      if (fatal == VRAI)
	break; /* sortie de la boucle sur Etape */
      /* erreur qui n'engendre pas un stop */
      } /* fin if (Code !=OK) */
    }  /* fin boucle sur Etape */

if (ip == 0)
  {
	    GNLControle.SortImp = SortImp_sv;
	    GNLControle.SortWE = SortWE_sv; 
  }

if (fatal == VRAI)
  return(Code);
else
  return(OK);

}

/* -------------- Fin fonction rego2C --------------------- */

