/* ----------------------------------------------------------------------
Ecriture sur fichier de l'instruction qui permet
de creer une structure Splus de type data-frame
 partir  des valeurs stockes dans Donnees
Il suffit  l'utilisateur de faire "source(nom-du-fichier-cr)", sous Splus,
pour crer une structure Splus de type data-frame.
------------------------------------------------------------------------- */

/* +++++++++++++++++++++++++++++++++++++++++++++++++++
les define et les includes 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include "nls2C.h"
#define precision 12 /* le nbre de chiffres significatifs sur le fichier cr */
/* pour crire les messages d'erreur */
extern void ecrErr(FILE *fic, char * nomcall , char *message, int Code);

/* Sp de ce module */
TLogic isnotun(TVect vect);
TLogic isnotmeme(TVectStr vect);

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
splusData
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
int 
splusData( char *nom , FILE *fic)

/* ............... Arguments d'entree      ....................... */
/* nom : le nom de la structure Splus a creer */
     /* fic: le  fichier de sortie */
/* ............... Arguments de sortie   ....................... */

     /* ............... Retour ....................................... */
     /* 0 ou un code d'erreur */
{
/* Pour les messages d'erreur */
char message[MAXLGMESS];
int nbx, obs;

/* ------------------ debut du corps ------------------------------ */

if (Donnees.NbX <= 0)
  {
    sprintf(message,
"Invalid value for the number of variables: %d:  have you called 'nls2C' or 'lect2C' before? \n", 
Donnees.NbX);
    ecrErr(stderr,"splus2C",message,  ERRFOUND);
	return((int)ERRFOUND);
  }

fprintf(fic, "%s <-  data.frame(%s=c(\n", nom, Donnees.NomY);
for (obs=0; obs < Donnees.NbObsT; obs++)
  {
    if (obs >0)
      fprintf(fic, ", ");
    fprintf(fic, "%.*g", precision,Donnees.ValY.donnees[obs]);
  }
    
fprintf(fic, "),\n");
for (nbx=0; nbx < Donnees.NbX; nbx++)
  {
    if (nbx >0)
      fprintf(fic, ", ");
    fprintf(fic, "%s =c(\n", Donnees.NomX.donnees[nbx]);
    for (obs=0; obs < Donnees.NbObsT; obs++)
      {
	if  (obs >0)
	  fprintf(fic, ", ");
	fprintf(fic, "%.*g", precision,Donnees.XObsT.donnees[obs][nbx]);
      }
    fprintf(fic, ")\n");
  }

if ((Donnees.PoidsT.nbele >0) && (isnotun(Donnees.PoidsT) != FAUX))
{
  fprintf(fic, ", weights =c(\n");
  for (obs=0; obs < Donnees.NbObsT; obs++)
      {
	if (obs >0)
	  fprintf(fic, ", ");
	fprintf(fic, "%.*g", precision,Donnees.PoidsT.donnees[obs]);
      }
fprintf(fic, ")\n");
}

if ((Donnees.NomCT.nbele > 0) && (isnotmeme(Donnees.NomCT)  != FAUX))
{
  fprintf(fic, ", curves = c(\n");
  for (obs=0; obs < Donnees.NbObsT; obs++)
      {
	if (obs >0)
	  fprintf(fic, ", ");
	fprintf(fic, "'%s'", Donnees.NomCT.donnees[obs]);
      }
fprintf(fic, ")\n");
}

fprintf(fic, ")\n");
return(OK);
}
/* ............ fin splusData ...................... */

/* -------------------------------------------------------
renvoie VRAI  si un TVect ne contient pas que des 1 (pour les poidsT)
----------------------------------------------------------*/
TLogic
isnotun(TVect vect)
{
int i;

for (i=0; i <vect.nbele; i++)
{
  if (vect.donnees[i] != (TDouble)1)
    return(VRAI);
}
return(FAUX);
}
/* ............ fin isnotun ...................... */


/* -------------------------------------------------------
renvoie VRAI  si un TVectStr ne contient pas que des valeurs identiques
 (pour les courbes, qui sont tries)
----------------------------------------------------------*/
TLogic
isnotmeme(TVectStr vect)
{
int i;

for (i=1; i <vect.nbele; i++)
{
  if (strcmp(vect.donnees[i], vect.donnees[i-1]) != 0)
    return(VRAI);
}
return(FAUX);
}
/* ............ fin isnotun ...................... */

